""" SchemaDefaults.py, automatically generated. 

    The STLDAPManager is designed to be highly configurable, but
configuring an instance can be a lot of work, which you will not
want to repeat.  This module, WHICH IS ONLY USED WHEN CREATING A NEW
LDAP Directory Manager, does provide a way to permanently record a
configuration.  The sensible way to configure a directory manager is
through-the-web.  When you are finished, you can save your configuration
in SchemaDefaults format, replacing this module.

    However, there are a few items that you may wish to configure the
first time, and they appear near the the top of the module.  Changing
things here can cause unit tests to fail, however.
"""

#   LDAP Server
#
#   Basic properties of the LDAP directory:
#
#     1. The server hostname and port (the port need not be specified
#        if it is the default 389).
#
#     2. The DN at the root of the tree to be managed by this instance.
#
#     3. The DN at the root of the tree to be searched when looking for
#        people.
#
#     4. The DN at the root of a tree containing organizational units
#        arranged hierarchically to match your org chart.  Children of
#        this DN should have 'organizationalUnit' as an objectClass 
#        value.
#
#     5. The DN at the root of a tree of groups.  Children of this DN
#        should have 'groupOfNames' or 'groupOfUniqueNames' as a value
#        of objectClass.
#
#     6. The DN at the root of a tree of groups used for determining
#        Zope roles.  Children of this DN should have 'groupOfNames'
#        or 'groupOfUniqueNames' as an objectClass value.
#
#     7. The scope of a search, starting at #6, for finding Zope roles
#        where:
#                 0 <=> 'Search the base ONLY'
#                 1 <=> 'Search one level of children'
#                 2 <=> 'Search the base and all subtrees'
#
#
#   Items #3 to #6 default to the value of #2.

defaultLDAPServer = ['ldap.stsci.edu', 'ldap2.stsci.edu']
defaultDirectoryBase = 'o=stsci.edu'
defaultPersonSearchBase = 'ou=People,o=stsci.edu'
defaultOrgUnitBrowseBase = 'ou=ST ScI,o=stsci.edu'
defaultGroupBrowseBase = 'ou=Groups,o=stsci.edu'
defaultZopeRoleSearchBase = 'ou=Groups,o=stsci.edu'
defaultZopeRoleSearchScope = 2

#  If you are using the SunONE/iPlanet/Netscape directory server
#   enter the ACIs on the directory root object.  This will allow
#   the directory manager to generate an LDIF file that sets all
#   of the ACIs for the directory.  (Modifying the ACI attribute
#   doesn't work, because the directory manager does not know
#   what the ACI to be replaced looks like.  So you have to 
#   delete the attribute and add ACIs back in.)  Preceed each
#   ACI with the line (don't indent):
#
#   add: aci
#
#   and separate ACIs with a line consisting only of a hyphen, as
#   in (don't indent):
#
#    add:aci
#    aci:
#     (target = "ldap:///o=test")
#     (targetattr = "*")
#     (version 3.0;
#     acl "Configuration Administrator";
#     allow (all)
#     (userdn = "ldap:///uid=admin,ou=Administrators, ou=TopologyManagement,
#     o=NetscapeRoot");)
#    -
#    add:aci
#    aci:
#     (target = "ldap:///o=test")
#     (targetattr = "*")
#     (version 3.0;
#     acl "Configuration Administrators Group";
#     allow (all)
#     (groupdn = "ldap:///cn=Configuration Administrators, ou=Groups, 
#     ou=TopologyManagement,o=NetscapeRoot");)

defaultIPlanetACIs = """add: aci
aci:
 (targetattr = "*")(version 3.0;acl "Configuration Adminstrator"; allow (all)userdn = "ldap:///uid=admin, ou=Administrators, ou=TopologyManagement, o=NetscapeRoot";)
-
add: aci
aci:
 (targetattr ="*")(version 3.0;acl "Configuration Administrators Group"; allow (all)(groupdn = "ldap:///cn=Configuration Administrators, ou=Groups, ou=TopologyManagement, o=NetscapeRoot");)
-
add: aci
aci:
 (targetattr = "*")(targetfilter = (employeetype=inactive)) (version 3.0;acl "Hide Inactive Persons from Anonymous Users"; deny (all)(userdn= "ldap:///anyone" and userdn!= "ldap:///all");)
-
add: aci
aci:
 (targetattr = "*")(version 3.0;acl "Directory Administrators"; allow (all)(userdn = "ldap:///uid=*,ou=Directory Administrators, o=stsci.edu");)
"""

#   Configured roles map to group names or acquired Zope role names, and
#   canonical roles are permanent roles that map to categories of
#   directory access: 

defaultConfiguredRoleNames = ['Divisional Administrators', 'System Administrators', 'Internal']
defaultCanonicalRoleNames = ['SELF', 'ANON', 'AUTH', 'MNGR']

#   personsOUAttr is the key for determing a person's org unit: 'dn' if
#   you look at a person's dn, or the name of the person attribute that
#   contains the person's org unit.
#   groupObjectClass can be either 'groupOfNames' or 'groupOfUniqueNames'.

defaultPersonsOUAttr = 'ou'
defaultGroupObjectClass = 'groupOfUniqueNames'

#   Controllers are the roles that can add/delete entries of a given
#   object category.

defaultGroupControllers = ['System Administrators']
defaultOrgUnitControllers = ['Divisional Administrators']
defaultUserControllers = ['System Administrators']

#   Roles that are not controllers may nevertheless have permission to
#   view tabs.  These are the roles that have access to the tabs.

defaultManagePeopleRoles = ['Divisional Administrators', 'System Administrators']
defaultManageGroupRoles = ['System Administrators']
defaultManageOrgUnitRoles = ['Divisional Administrators']
defaultManageSchemaRoles = ['Manager']

#   The permanentRoles dictionary contains a configurable roleName
#   and a description for each role.

defaultPermanentRoles = {

        'SELF' : {
                'roleName'     : 'Self',
                'description'  : """
                                 The role controlling access to the
                                 authenticated user's own record.
                                 """ },

        'ANON' : {
                'roleName'     : 'Anonymous User',
                'description'  : """
                                 The role controlling access by an
                                 anonymous (i.e., un-authenticated) user.
                                 """ },

        'AUTH' : {
                'roleName'     : 'Authenticated User',
                'description'  : """
                                 The role controlling an authenticated
                                 user's access to the objects in the
                                 directory, in general.
                                 """ },

        'MNGR' : {
                'roleName'     : 'Direct Manager',
                'description'  : """
                                 The role controlling access to a user's
                                 record by that user's direct manager.
                                 (Not the Zope manager role.)
                                 """ },
        }

#   DisplayCategories are selections of object attributes grouped onto a
#   single display form.  Use them to split up object display into chunks
#   that make sense for you.

defaultGroupDisplayCategories = ['General', 'System']
defaultOrgUnitDisplayCategories = ['General', 'Telephone', 'Postal Mail', 'E-Mail', 'Organization', 'System']
defaultUserDisplayCategories = ['General', 'Personal', 'Telephone', 'Postal Mail', 'E-Mail', 'Organization', 'System', 'All']

#   The namingAttribute, or relative distinguished name (RDN) for each
#   object category is the name of the attribute used to name new objects.

defaultGroupNamingAttribute = 'cn'
defaultOrgUnitNamingAttribute = 'ou'
defaultUserNamingAttribute = 'uid'

#   These parameters control the result of searches for objects.  The
#   filter must narrow results to the right category of object, and
#   could include other tests (we filter out inactive employees).  The
#   search keys are the names of the attributes searched for a search
#   string, and are also the attributes returned and displayed.  The
#   searchSortKey is the name of the attribute used to sort the results.
#   The browseFormat is a format string for formatting entries shown in
#   a tree view of the directory.  The browseKeys are attribute names
#   formatted according to the formatString, and the sortKey is for
#   sorting entries under each node of the tree.

defaultGroupSearchFilter = "(|(objectclass=groupofuniquenames)(objectclass=groupofnames))"
defaultGroupSearchKeys = ['cn']
defaultGroupSearchSortKey = 'cn'
defaultGroupBrowseFormat = '%s'
defaultGroupBrowseKeys = []
defaultGroupBrowseSortKey = 'None'

defaultOrgUnitSearchFilter = "(objectclass=organizationalunit)"
defaultOrgUnitSearchKeys = ['ou', 'manager', 'stsciAdministrator']
defaultOrgUnitSearchSortKey = 'ou'
defaultOrgUnitBrowseFormat = '%s'
defaultOrgUnitBrowseKeys = []
defaultOrgUnitBrowseSortKey = 'None'

defaultUserSearchFilter = "(objectclass=person)(!(employeetype=inactive))"
defaultUserSearchKeys = ['displayName', 'telephoneNumber', 'roomNumber', 'departmentNumber', 'mail', 'physicalDeliveryOfficeName', 'uid']
defaultUserSearchSortKey = 'displayName'
defaultUserBrowseFormat = '%s'
defaultUserBrowseKeys = ['displayName']
defaultUserBrowseSortKey = 'displayName'

#   The pointsToTargets are the options in the pointsTo pull-down
#   menu for managing attributes that says what object category is the
#   value of an attribute whose format is a distinguished name.  This
#   controls the interface to the attribute values, and can't be
#   extended without writing code.

defaultPointsToTargets = ['', 'person', 'orgUnit', 'group']

defaultGroupSchema = {

        'cn' : {
                'friendlyName' : 'Group Name',
                'description' : """The group's common name.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators'],
                'displayCategories' : ['General', 'System'],
                'required' : 1, },

        'description' : {
                'friendlyName' : 'Description',
                'description' : """Description of the group.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'System Administrators'],
                'displayCategories' : ['General', 'System'], },

        'o' : {
                'friendlyName' : 'Organization',
                'description' : """The organization to which the group belongs.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators'],
                'displayCategories' : ['System'], },

        'objectClass' : {
                'friendlyName' : 'Object Class',
                'description' : """Defines the kind of object referring to.  Each object class has its own schema.  Each object class defines a list of attributes that are required and defines the attributes that are allowed.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators'],
                'displayCategories' : ['System'],
                'validValues' : ['groupOfUniqueNames', 'top'],
                'requiredValues' : ['groupOfUniqueNames', 'top'],
                'required' : 1, },

        'ou' : {
                'friendlyName' : 'Organizational Unit',
                'description' : """The name of the group's organizational unit.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators'],
                'displayCategories' : ['System'], },

        'owner' : {
                'friendlyName' : 'Owner',
                'description' : """The distinguished name of the group's owner.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'System Administrators'],
                'displayCategories' : ['System'],
                'pointsTo' : 'person', },

        'seeAlso' : {
                'friendlyName' : 'See Also',
                'description' : """Distinguished name of a related group.""",
                'readers' : ['SELF', 'AUTH', 'ANON', 'System Administrators'],
                'writers' : ['SELF', 'System Administrators'],
                'displayCategories' : ['System', 'General'],
                'pointsTo' : 'group', },

        'uniqueMember' : {
                'friendlyName' : 'Members',
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'System Administrators'],
                'displayCategories' : ['System', 'General'],
                'pointsTo' : 'person', },
        }

defaultOrgUnitSchema = {

        'description' : {
                'friendlyName' : 'Description',
                'description' : """Text description of the person.  Provides a human-readable description of the object. For people or organizations, this often includes their role or work assignment.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['General'], },

        'facsimileTelephoneNumber' : {
                'friendlyName' : 'FAX Number',
                'description' : """The person's FAX number""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Telephone'], },

        'manager' : {
                'friendlyName' : 'Manager',
                'description' : """Distinguished name representing the org unit's manager.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Organization'],
                'pointsTo' : 'person', },

        'objectClass' : {
                'friendlyName' : 'Object Class',
                'description' : """Defines the kind of object referring to.  Each object class has its own schema.  Each object class defines a list of attributes that are required and defines the attributes that are allowed.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['System'],
                'validValues' : ['STScIOrganizationalUnit', 'organizationalUnit', 'top'],
                'requiredValues' : ['STScIOrganizationalUnit', 'organizationalUnit', 'top'],
                'required' : 1, },

        'ou' : {
                'friendlyName' : 'Organizational Unit Name',
                'description' : """The distinguished name of the organizational unit to which the person belongs.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['General'],
                'required' : 1, },

        'physicalDeliveryOfficeName' : {
                'friendlyName' : 'Mailstop',
                'description' : """The org unit's STSCI location for receiving postal mail.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Postal Mail', 'General'], },

        'postOfficeBox' : {
                'friendlyName' : 'Post Office Box',
                'description' : """The org unit's post office box""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Postal Mail'], },

        'postalAddress' : {
                'friendlyName' : 'Postal Address',
                'description' : """Identifies the entry's mailing address. This field is intended to include multiple lines. """,
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Postal Mail'], },

        'postalCode' : {
                'friendlyName' : 'Postal Code',
                'description' : """The org unit's postal code.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Postal Mail'], },

        'st' : {
                'friendlyName' : 'State',
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Postal Mail'], },

        'street' : {
                'friendlyName' : 'Street Address',
                'description' : """Identifies the entry's house number and street name.  For example:\n
streetAddress: 1234 Ridgeway Drive""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Postal Mail'], },

        'stsciAdministrator' : {
                'friendlyName' : 'Divisional Administrator',
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Organization'],
                'pointsTo' : 'person', },

        'telephoneNumber' : {
                'friendlyName' : 'Telephone Number',
                'description' : """The org unit's telephone number""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Telephone'], },
        }

defaultUserSchema = {

        'businessCategory' : {
                'friendlyName' : 'Category',
                'description' : """Type of business.  Business in which the organizational unit is involved.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Organization', 'All'], },

        'carLicense' : {
                'friendlyName' : 'Automobile License',
                'description' : """The license plate number of the person's vehicle.""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['SELF', 'Divisional Administrators'],
                'displayCategories' : ['Personal', 'All'],
                'requiredValues' : [''], },

        'cn' : {
                'friendlyName' : 'Common Name(s)',
                'description' : """The person's common name.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['General', 'Personal', 'All'],
                'required' : 1, },

        'departmentNumber' : {
                'friendlyName' : 'Branch',
                'description' : """The number for the department for which the person works.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['General', 'All'], },

        'description' : {
                'friendlyName' : 'Description',
                'description' : """Text description of the person.  Provides a human-readable description of the object. For people or organizations, this often includes their role or work assignment.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'Divisional Administrators'],
                'displayCategories' : ['General', 'All'], },

        'displayName' : {
                'friendlyName' : 'Display Name',
                'description' : """Preferred name of a person to be used when displaying entries. Especially useful for displaying a preferred name for an entry within a one-line summary list.  Since other attribute types, such as cn, are multi-valued, they cannot be used to display a preferred name.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['General', 'All'], },

        'employeeType' : {
                'friendlyName' : 'Employee Type',
                'description' : """The person's type of employee (for example, full-time)""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Organization', 'All'],
                'validValues' : ['Full-Time', 'Part-Time', 'Intern', 'Contractor', 'Consultant', 'Affiliate', 'Inactive'],
                'requiredValues' : [''], },

        'facsimileTelephoneNumber' : {
                'friendlyName' : 'FAX Number',
                'description' : """The person's fax number""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators', 'SELF'],
                'displayCategories' : ['Telephone', 'All'], },

        'givenName' : {
                'friendlyName' : 'Given Name',
                'description' : """The person's given or first name""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Personal', 'All'], },

        'homePhone' : {
                'friendlyName' : 'Home Phone',
                'description' : """The person's home phone number""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['SELF', 'Divisional Administrators'],
                'displayCategories' : ['Personal', 'All'],
                'requiredValues' : [''], },

        'homePostalAddress' : {
                'friendlyName' : 'Home Address',
                'description' : """The person's home address.""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['SELF', 'Divisional Administrators'],
                'displayCategories' : ['Personal', 'All'],
                'requiredValues' : [''], },

        'initials' : {
                'friendlyName' : 'Initials',
                'description' : """The person's middle initial(s).""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Personal', 'All'], },

        'jpegPhoto' : {
                'friendlyName' : 'Photo',
                'description' : """JPEG format photo""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Personal', 'All'], },

        'l' : {
                'friendlyName' : 'Location',
                'description' : """Identifies the county, city, or other geographical area in which the entry is located, or in some other way, associated with.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Postal Mail', 'All'], },

        'labeledUri' : {
                'friendlyName' : 'Home Page',
                'description' : """Specifies a Uniform Resource Identifier (URI) that is relevant in some way to the entry. Values placed in the attribute should consist of a URI (currently only URLs are supported) optionally followed by one or more space characters and a label.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators', 'Divisional Administrators'],
                'displayCategories' : ['General', 'All'], },

        'mail' : {
                'friendlyName' : 'Email Address',
                'description' : """Identifies a user's primary email address.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators'],
                'displayCategories' : ['E-Mail', 'General', 'All'], },

        'mailAlternateAddress' : {
                'friendlyName' : 'Mail Published Address',
                'description' : """The person's alternate email address.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators'],
                'displayCategories' : ['E-Mail', 'All'], },

        'mailHost' : {
                'friendlyName' : 'Mail Host',
                'description' : """The mail server where the person's mailbox resides.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators'],
                'displayCategories' : ['E-Mail', 'All'], },

        'mailRoutingAddress' : {
                'friendlyName' : 'Mail Routing Address',
                'description' : """The person's internal mail delivery address for routing incoming email.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators'],
                'displayCategories' : ['E-Mail', 'All'], },

        'manager' : {
                'friendlyName' : 'Manager',
                'description' : """Distinguished name representing the person's manager""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Organization', 'All'],
                'pointsTo' : 'person', },

        'miService' : {
                'description' : """Specifies the Mirapoint service. (Currently used for testing purposes.)""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators', 'Divisional Administrators'],
                'displayCategories' : ['E-Mail'],
                'validValues' : ['antispam'], },

        'mobile' : {
                'friendlyName' : 'Cell Phone',
                'description' : """The number of the person's cell phone.""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['SELF', 'Divisional Administrators'],
                'displayCategories' : ['Telephone', 'All'],
                'requiredValues' : [''], },

        'o' : {
                'friendlyName' : 'Organization',
                'description' : """The distinguished name of the organization to which the person belongs.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Organization', 'All'], },

        'objectClass' : {
                'friendlyName' : 'Object Class',
                'description' : """Defines the kind of object referring to.  Each object class has its own schema.  Each object class defines a list of attributes that are required and defines the attributes that are allowed.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators'],
                'displayCategories' : ['System', 'All'],
                'validValues' : ['STScIPerson', 'inetOrgPerson', 'organizationalPerson', 'person', 'top', 'mailRecipient'],
                'requiredValues' : ['STScIPerson', 'inetOrgPerson', 'organizationalPerson', 'person', 'top', 'mailRecipient'],
                'required' : 1, },

        'ou' : {
                'friendlyName' : 'Organizational Unit',
                'description' : """The distinguished name of the organizational unit to which the person belongs.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Organization', 'All'], 
		'pointsTo' : 'orgUnit', },

        'pager' : {
                'friendlyName' : 'Pager',
                'description' : """The person's pager number.""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['SELF', 'Divisional Administrators'],
                'displayCategories' : ['Telephone', 'All'],
                'requiredValues' : [''], },

        'physicalDeliveryOfficeName' : {
                'friendlyName' : 'Mail Stop',
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Postal Mail', 'All'], },

        'postOfficeBox' : {
                'friendlyName' : 'Post Office Box',
                'description' : """The person's post office box.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Postal Mail', 'All'], },

        'postalAddress' : {
                'friendlyName' : 'Postal Address',
                'description' : """The person's postal address.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Postal Mail', 'All'], },

        'postalCode' : {
                'friendlyName' : 'Postal Code',
                'description' : """The person's postal code.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Postal Mail', 'All'], },

        'roomNumber' : {
                'friendlyName' : 'Office Number',
                'description' : """Specifies the person's room number.""",
                'readers' : ['AUTH', 'Internal'],
                'writers' : ['SELF', 'Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['General', 'All'], },

        'secretary' : {
                'friendlyName' : 'Divisional Administrator',
                'description' : """Identifies the entry's Divisional Administrator.""",
                'readers' : ['AUTH', 'Internal'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Organization', 'All'],
                'pointsTo' : 'person', },

        'sn' : {
                'friendlyName' : 'Last Name',
                'description' : """Identifies the person's last or family name.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['SELF', 'Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['General', 'All'],
                'required' : 1, },

        'st' : {
                'friendlyName' : 'State',
                'description' : """Identifies the state or province in which the person resides.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Postal Mail', 'All'], },

        'street' : {
                'friendlyName' : 'Street Address',
                'description' : """Identifies the person's house number and street name.  For example:\n streetAddress: 1234 Ridgeway Drive""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Postal Mail', 'All'], },

        'stsciAccessCardNumber' : {
                'friendlyName' : 'Access Card Number(s)',
                'description' : """The person's security access card number(s).""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Organization', 'All'],
                'requiredValues' : [''], },

        'stsciContactAddress' : {
                'friendlyName' : 'Contact Address',
                'description' : """Emergency contact address.""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['SELF', 'Divisional Administrators'],
                'displayCategories' : ['Personal', 'All'],
                'requiredValues' : [''], },

        'stsciContactName' : {
                'friendlyName' : 'Contact Name',
                'description' : """Emergency contact name(s).""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['SELF', 'Divisional Administrators'],
                'displayCategories' : ['Personal', 'All'],
                'requiredValues' : [''], },

        'stsciContactPhone' : {
                'friendlyName' : 'Contact Phone',
                'description' : """Emergency contact phone number(s).""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['SELF', 'Divisional Administrators'],
                'displayCategories' : ['Personal', 'All'],
                'requiredValues' : [''], },

        'stsciKeyCodes' : {
                'friendlyName' : 'Key Codes',
                'description' : """The alpha/numeric code stamped on the person's office door key.""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Organization', 'All'],
                'requiredValues' : [''], },

        'stsciParkingSticker' : {
                'friendlyName' : 'Parking Sticker Number',
                'description' : """The person's parking sticker number(s).""",
                'readers' : ['SELF', 'MNGR', 'Divisional Administrators'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['Organization', 'All'],
                'requiredValues' : [''], },

        'telephoneNumber' : {
                'friendlyName' : 'Telephone Number',
                'description' : """The person's office telephone number(s), one per line.  When four digits are given, its the four digit extension number.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['Telephone', 'General', 'All'], },

        'title' : {
                'friendlyName' : 'Title',
                'description' : """The person's job title.""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['Divisional Administrators'],
                'displayCategories' : ['General', 'All'], },

        'uid' : {
                'friendlyName' : 'Username',
                'description' : """The entry's user ID (user name)""",
                'readers' : ['ANON', 'AUTH'],
                'writers' : ['System Administrators'],
                'displayCategories' : ['System', 'All'],
                'required' : 1, },

        'userPassword' : {
                'friendlyName' : 'User Password',
                'description' : """Enter a new password and confirm it.  If you change your own Password, you will be logged out and returned to the search screen, where you can login again using your new password.""",
                'readers' : ['SELF', 'System Administrators'],
                'writers' : ['SELF', 'System Administrators'],
                'displayCategories' : ['General', 'All'],
                'requiredValues' : [''], },
        }

