"""

    Module: $RCSfile: ZRoles.py,v $
    Author: $Author: payne $
    Date:   $Date: 2003/11/20 03:55:20 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""

__version__ = '$Revision: 1.16 $'[11:-2]

from LDAPUtilities import DNsAreEquivalent

from AccessControl.Permission import Permission
from AccessControl.Role import RoleManager
from Acquisition import Implicit
from Globals import DTMLFile
from string import index

class ZRoles( Implicit ):

    manage_rolesFormSelection = DTMLFile('dtml/rolesFormSelection', globals())
    _rolesRenameForm = DTMLFile('dtml/rolesRename', globals())
    _rolesSelectForm = DTMLFile('dtml/rolesSelect', globals())
    _rolesPermissionsForm = DTMLFile('dtml/rolesPermissions', globals())

    def __init__(self, user, request ):

        self.user = user
        self._managedRoles = user.getLDAPDirectory().getManagedRoles()
        self.request = request

    def DNsAreEquivalent( self, dn1, dn2):
        """
           Wrapper for LDAPUtilities.DNsAreEquivalent
        """
        return DNsAreEquivalent( dn1, dn2 )

    def getManagedRoles(self):
        """
            Returns a managed roles object.
        """
        return self._managedRoles

    def getSchemaPermission(self):
        """
        Returns the name of the permission for managing the schema
        """
        return self.manageSchemaPermission

    def permission_settings(self):
        """Return user-role permission settings
        """
        result=[]
        valid=self.valid_roles()
#        valid = self._managedRoles.getConfiguredRoleNames()
        indexes=range(len(valid))
        ip=0
        ldapPermissions = [ self.manageGroupPermission,
                            self.manageOrgUnitPermission,
                            self.managePeoplePermission,
                            self.manageSchemaPermission,]

        for p in self.ac_inherited_permissions(1):    
            name, value = p[:2]
            p=Permission(name,value,self.aq_parent)
            roles=p.getRoles(default=[])
            d={'name': name,
               'acquire': type(roles) is type([]) and 'CHECKED' or '',
               'roles': map(
                   lambda ir, roles=roles, valid=valid, ip=ip:
                   {
                       'name': "p%dr%d" % (ip,ir),
                       'checked': (valid[ir] in roles) and 'CHECKED' or '',
                       },
                   indexes)
               }
            ip=ip+1
            if name in ldapPermissions:
                result.append(d)
        return result

    def showRolesForm( self, message ):
        """
            Given a request for a role management form, invoke any
            role updates that may have been called for in the request,
            and then return the appropriate form.
        """

        #   If we got here by a form submit button, then we need to
        #   update the schema first.

        if self.request.has_key('updateSchema'):
            if not self.request.has_key('cancel'):
                message = self.updateRolesFromRequest()
                self.aq_parent._p_changed = 1
                parent = self.aq_parent
                self = ZRoles( self.user, self.request )
                self = self.__of__( parent )

        #   Set default form, and render it.  Each form must contain a
        #   hidden tag called 'rolesFormID' whose value, with '_'
        #   prepended, and 'Form' appended, is form rendering DTMLFile
        #   call.

        if not self.request.has_key('rolesFormID'):
            self.request.set('rolesFormID','rolesSelect')

        if self.request.rolesFormID == 'rolesPermissions':
            return self._rolesPermissionsForm( 
                        self, self.request, manage_tabs_message = message )

        elif self.request.rolesFormID == 'rolesRename':
            return self._rolesRenameForm( 
                        self, self.request, manage_tabs_message = message )

        elif self.request.rolesFormID == 'rolesSelect':
            return self._rolesSelectForm(
                        self, self.request, manage_tabs_message = message )

        else:
            message = """
                      The previous form's rolesFormID, %s, does not match a
                      known form display method.
                      """ % self.request.rolesFormID
            return self._schemaManagedForm( self, self.request,
                                            manage_tabs_message = message )

    def updateRolesFromRequest( self ):
        """
            Change the roles according to the REQUEST variables.
        """
        message = None

        #   Invoke a method that matches the name of the submit
        #   button (with '_' prepended).

        if self.request.has_key( 'addRoles' ):
            message = self._addRoles()

        elif self.request.has_key( 'changePermissions' ):
            message = self._changePermissions()

        elif self.request.has_key( 'deleteConfiguredRoles' ):
            message = self._deleteRoles()

        elif self.request.has_key( 'renamePermanentRoles' ):
            message = self._renameRoles()

        else:
            # You can get here if your form has a single text input
            # field and you press return rather than the form button.
            # Maybe there is a way to figure it out:

            message = "No method to perform requested action on roles."

        return message

    def _addRoles(self):

        # Add configured roles, given a list of names.

        roleNames = self.request.get( 'newRoles', [] )

        for role in roleNames:
            self._managedRoles.addRole( role )
            RoleManager._addRole( self.aq_parent, role )

	if roleNames:
            return "Added roles."
	else:
	    return ""

    def _changePermissions(self):

        # Wrap the RoleManager manage_changePermissions

	# For readability, we suppressed the Acquire Permission 
	# Settings? column in the display, so those request items
	# are mission.  Put them back.

        permissions=self.ac_inherited_permissions(1)
        indexes=range(len(permissions))
        for i in indexes:
	    key = "a%s" % ( i )
	    self.request.form[ key ] = 1
        RoleManager.manage_changePermissions( self.aq_parent, self.request )

    def _deleteRoles(self):

        # Delete configured roles, given the names

        roleNames = self.request.get( 'ids', [] )

        # Easy: delete from the list

        for role in roleNames:
            self._managedRoles.delRole( role )
        self.aq_parent._delRoles( roleNames, None )

        # Hard: look through all of the attribute reader and writer
        # lists in all of the schema and delete from them.

        ldapDirectory = self.user.getLDAPDirectory()
        for schema in (ldapDirectory.getPersonSchema(), 
                       ldapDirectory.getOrgUnitSchema(),
                       ldapDirectory.getGroupSchema() ):

            for ldapName in schema.getAllowedAttributeNames():
                schema.getAttributeSchema(ldapName).delReaders( roleNames )
                schema.getAttributeSchema(ldapName).delWriters( roleNames )

	if roleNames:
            return "Deleted role(s)."
	else:
	    return ""

    def _changeRoles(self):

        # A method for updating the roles list.

        option = self.request.get( 'changeRoles' )
        ids = self.request.get( 'ids' )

        if option == " Delete " and len( ids ) > 0:
            for item in ids:
                if item in self._managedRoles.getConfiguredRoleNames():
                    self._managedRoles.delRole( item )
            message = "Deleted configured roles."

        elif option == " Add ":
            newRole = self.request.get( 'newRole', None )
            if newRole:
                self._managedRoles.addRole( newRole )
            message = "Added role."

        else:
            message = ""

        return message

    def _renameRoles(self):
        """
        Rename roles.
        """
        #   If we got here from the _schemaRenameRolesForm, then 
        #   perform the rename.

        if self.request.has_key('haveNewNames'):

            canonicalNames = self.request.get( 'ids', [] )
            newNames = self.request.get( 'new_ids', [] )

            # Skip empty new_ids

            while '' in newNames:
                i = index(newNames,'')
                canonicalNames.pop(i)
                newNames.pop(i)

            for i in range( 0, len( canonicalNames ) ):
                canonicalName = canonicalNames[ i ]
                newName = newNames[ i ]

                self._managedRoles._permanentRoles[canonicalName]['roleName'] =\
                    newName

            return "Renamed roles categories.  " 

        #   Otherwise, set REQUEST variables so that the
        #   _schemaRenameCategoriesForm gets displayed.

        else:
            canonicalNames = self.request.get( 'ids', [] )
	    if canonicalNames:
                self.request.set('rolesFormID', 'rolesRename')
		if self.request.form.has_key('updateSchema'):
                    del self.request.form['updateSchema']

            return ""


