"""

    Module: $RCSfile: TestLDIFObjectParser.py,v $
    Author: $Author: payne $
    Date:   $Date: 2003/07/25 19:12:47 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.    

"""
__version__ = '$Revision: 1.4 $'[11:-2]

import sys
sys.path.insert(0,'..')

import unittest
from LDIFObjectParser import LDIFObjectParser

class testLDIFObjectParser(unittest.TestCase):

    def setUp(self):
        pass
    
    def tearDown(self):
        pass

    def test01(self):
        """ 01 testLDIFObjectParser: testing top """
        ldif = "( 2.5.6.0 NAME 'top' DESC 'Standard LDAP objectclass' " \
             + "ABSTRACT MUST objectClass X-ORIGIN 'RFC 2256' )"
        obj = LDIFObjectParser(ldif)
        self.failUnless( obj.oid == '2.5.6.0' )
        self.failUnless( obj.name == 'top' )
        self.failUnless( obj.description == 'Standard LDAP objectclass' )
        self.failUnless( obj.objectType == 'ABSTRACT' )
        self.failUnless( obj.mustContain == ['objectClass'])
        self.failUnless( obj.mayContain == [] )
        self.failUnless( obj.origin == 'RFC 2256' )
        self.failUnless( ldif == obj.getLDIF())
        
    def test02(self):
        """ 02 testLDIFObjectParser: testing alias """
        ldif = "( 2.5.6.1 NAME 'alias' DESC 'Standard LDAP objectclass' " \
             + "SUP top ABSTRACT MUST aliasedObjectName X-ORIGIN 'RFC 2256' )"
        self.failUnless( ldif == LDIFObjectParser(ldif).getLDIF())
 
    def test03(self):
        """ 03 testLDIFObjectParser: testing auxiliary """
        ldif = "( 1.3.6.1.4.1.1466.101.120.111 NAME 'extensibleObject' " \
             + "DESC 'LDAPv3 extensible object' SUP top AUXILIARY " \
             + "X-ORIGIN 'RFC 2252' )"
        self.failUnless( ldif == LDIFObjectParser(ldif).getLDIF())

    def test04(self):
        """ 04 testLDIFObjectParser: testing organizationalUnit """
        ldif = "( 2.5.6.5 NAME 'organizationalUnit' " \
             + "DESC 'Standard LDAP objectclass' SUP top STRUCTURAL " \
             + "MUST ou MAY ( businessCategory $ description " \
             + "$ destinationIndicator $ facsimileTelephoneNumber " \
             + "$ internationaliSDNNumber $ l $ physicalDeliveryOfficeName " \
             + "$ postOfficeBox $ postalAddress $ postalCode " \
             + "$ preferredDeliveryMethod $ registeredAddress $ searchGuide " \
             + "$ seeAlso $ st $ street $ telephoneNumber " \
             + "$ teletexTerminalIdentifier $ telexNumber $ userPassword " \
             + "$ x121Address ) X-ORIGIN 'RFC 2256' )"
        self.failUnless( ldif == LDIFObjectParser(ldif).getLDIF())

    def test05(self):
        """ 05 testLDIFObjectParser: testing organizationalPerson """
        ldif = "( 2.5.6.7 NAME 'organizationalPerson' " \
             + "DESC 'Standard LDAP objectclass' SUP person STRUCTURAL " \
             + "MAY ( destinationIndicator $ facsimileTelephoneNumber " \
             + "$ internationaliSDNNumber $ l $ ou " \
             + "$ physicalDeliveryOfficeName $ postOfficeBox " \
             + "$ postalAddress $ postalCode $ preferredDeliveryMethod " \
             + "$ registeredAddress $ st $ street " \
             + "$ teletexTerminalIdentifier $ telexNumber $ title " \
             + "$ x121Address ) X-ORIGIN 'RFC 2256' )"
        self.failUnless( ldif == LDIFObjectParser(ldif).getLDIF())

if __name__ == '__main__':
    
    tests = unittest.makeSuite(testLDIFObjectParser, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
