"""

    Module: $RCSfile: TestSTLDAPManager.py,v $
    Author: $Author: payne $
    Date:   $Date: 2003/12/15 14:55:19 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.4 $'[11:-2]

import os, sys
import ZopeEnviron

import Zope
import ZODB
import ZServer
import ZPublisher

from AccessControl import ClassSecurityInfo, getSecurityManager
from AccessControl.Role import RoleManager
from Acquisition import Implicit
from Globals import DTMLFile, InitializeClass
from OFS.ObjectManager import ObjectManager
from ZPublisher.Request import Request
from ZPublisher.Response import Response

from copy import deepcopy
from string import index, split
from StringIO import StringIO
import unittest

from Connection import AuthenticatedUser
from LDAPExceptions import REQUIRED_ATTRIBUTE, NO_SUCH_ATTRIBUTE
from LDAPUtilities import listsAreEquivalent
from ManagedRoles import ManagedRoles
from ManagedSchema import ManagedSchema
from SchemaDefaults_test import *
from STLDAPManager import STLDAPManager, addFunction
from TestDirectory import *
from ZReports import ZReports
from ZRoles import ZRoles
from ZSchema import ZSchema

app = Zope.app()

class testSTLDAPManager(unittest.TestCase,STLDAPManager,ObjectManager):

    managePeoplePermission = STLDAPManager.managePeoplePermission
    manageGroupPermission = STLDAPManager.manageGroupPermission
    manageOrgUnitPermission = STLDAPManager.manageOrgUnitPermission
    manageSchemaPermission = STLDAPManager.manageSchemaPermission

    __ac_roles__ = ('Divisional Administrator', 'System Administrator' )

    __ac_permissions__ = (
	    (managePeoplePermission,
	      ()),
	    (manageGroupPermission,
	      ()),
	    (manageOrgUnitPermission,
	      ()),
	    (manageSchemaPermission,
	      ()),
	    )

    security = ClassSecurityInfo()
    security.setPermissionDefault(
	managePeoplePermission,
	('Manager', 'Divisional Administrator'),
	)
    security.setPermissionDefault(
	manageGroupPermission,
	('Manager', 'Divisional Administrator'),
	)
    security.setPermissionDefault(
	manageOrgUnitPermission,
	('Manager', 'Divisional Administrator'),
	)
    security.declareProtected(
	managePeoplePermission,'setUp')
    security.declareProtected(
	manageGroupPermission,'dummy1')
    security.declareProtected(
	manageOrgUnitPermission,'dummy2')

    def dummy1(self):
	pass
    def dummy2(self):
	pass

    class mySTLDAPManager(STLDAPManager):
	# Wrapper for STLDAPManager so we can tell it where the form 
	# templates are
	def __init__(self, user, REQUEST ):
	    STLDAPManager.__init__(self, user, REQUEST )
        def __init__( self, id, title, ldapServer, directoryBase,
		personSearchBase, orgUnitBrowseBase, groupBrowseBase, 
		zopeRoleSearchBase, zopeRoleSearchScope, 
		personNamingAttribute, personsOUAttr, givenAttr, 
		groupObjectClass, REQUEST=None ):
	    STLDAPManager.__init__( self, id, title, ldapServer, directoryBase,
		personSearchBase, orgUnitBrowseBase, groupBrowseBase, 
		zopeRoleSearchBase, zopeRoleSearchScope, 
		personNamingAttribute, personsOUAttr, givenAttr, 
		groupObjectClass, REQUEST=None )

	abs = os.environ['PWD']+'/../' 
	manage_configureForm = DTMLFile(abs+'dtml/configure', globals())
	orgChartForm = DTMLFile(abs+'dtml/orgChart', globals())
	searchForm = DTMLFile(abs+'dtml/search', globals())
	viewForm = DTMLFile(abs+'dtml/view', globals())

    def Destination(self):
	return self

    def setUp(self):
	pass

    def test01(self):
        """ 01 TestSTLDAPManager: test instantiation """
        get_transaction().begin()
	self = self.__of__( app )
	self.__name__ = 'test'
	self.manage_addFolder('testFolder')

        directory = getTestDirectory()
        self.user = AuthenticatedUser('testadmin', 'testadmin',
                       directory)
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
        zRoles = ZRoles( self.user, REQUEST )
        self.testFolder.zRoles = zRoles.__of__( self)

	REQUEST.set('objectCategory','person')
        zSchema = ZSchema( self.user, REQUEST )
	self.testFolder.zSchema = zSchema.__of__(self)
	self.testFolder.zSchema._assignRolesToPermission()

	REQUEST.set('objectCategory','orgUnit')
        zSchema = ZSchema( self.user, REQUEST )
	self.testFolder.zSchema = zSchema.__of__(self)
	self.testFolder.zSchema._assignRolesToPermission()

	REQUEST.set('objectCategory','group')
        zSchema = ZSchema( self.user, REQUEST )
	self.testFolder.zSchema = zSchema.__of__(self)
	self.testFolder.zSchema._assignRolesToPermission()

	manager = self.mySTLDAPManager( 'testManager', '',
		ldapServer,
		defaultDirectoryBase,
		defaultPersonSearchBase,
		defaultOrgUnitBrowseBase,
		defaultGroupBrowseBase,
		defaultZopeRoleSearchBase,
		defaultZopeRoleSearchScope,
		defaultUserNamingAttribute,
		'useGivenAttr',
		'ou',
		defaultGroupObjectClass,
		REQUEST=None )

        REQUEST = Request(sys.stdin, os.environ, self.response )
	manager.LDAPDirectory = self.user.getLDAPDirectory()
	manager.REQUEST = REQUEST
	self.testFolder.manager = manager.__of__(self.testFolder)
	self.testFolder = self.testFolder.__of__(self)
	self = self.__of__(app)
        response = self.testFolder.manager.index_html( REQUEST )
        self.failUnless(index(response,'Search for:')>0)
        get_transaction().abort()
        app._p_jar.close()

    def test02(self):
        """ 02 TestSTLDAPManager: testing index_html """

	f=StringIO()
	temp = sys.stdout
	sys.stdout = f
	ZPublisher.Zope('/openldap/manager',u='testadmin:testadmin',)
#		e={"SERVER_URL":"http://%s" % ldapServer[0],})
	sys.stdout = temp
        self.failUnless(index(f.getvalue(),'Search for:')>0)

    def tearDown(self):
	pass

InitializeClass(testSTLDAPManager)

if __name__ == '__main__':

    tests = unittest.makeSuite(testSTLDAPManager, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
