"""

    Module: $RCSfile: TestZLDAPTree.py,v $
    Author: $Author: payne $
    Date:   $Date: 2003/11/16 03:23:30 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.1 $'[11:-2]


import os, sys
import ZopeEnviron

import Zope
import ZServer
#sys.path.insert(0,'..')

from Connection import *
from LDAPUtilities import _ldap
from LDAPUtilities import *
from ManagedSchema import ManagedSchema
from RequestTest import request
from SchemaDefaults_test import *
from TestDirectory import getTestDirectory
from ZLDAPTree import ZLDAPTree

from string import index
import unittest

class testZLDAPTree(unittest.TestCase):
    """ Tests the ZLDAPTree class """

    def setUp(self):
        self.directory = getTestDirectory()
        groupSchema = ManagedSchema( defaultGroupSchema, 
                                     defaultGroupDisplayCategories,
                                     defaultGroupNamingAttribute,
                                     defaultGroupControllers,
                                     defaultGroupSearchFilter,
                                     defaultGroupSearchKeys,
                                     defaultGroupSearchSortKey)
        self.directory._groupManagedSchema = groupSchema
        self.REQUEST = request()
#        self.user = AnonymousUser(self.directory)
        self.user = AuthenticatedUser('testadmin', 'testadmin',
                       self.directory)
        self.ldapTree = ZLDAPTree()

    def test01(self):
        """ 01 testZLDAPTree: getNodeDict """
        base = "o=test"
        groupSchema = self.directory._groupManagedSchema
        filter = "(objectClass=organizationalUnit)"
        dict = self.ldapTree.getNodeDict( self.user, base, groupSchema,
					  filter=filter )
        self.failUnless( dict['base'].mappingDN == 'o=test' )
        base = "ou=Groups,o=test"
        filter = "(|(objectClass=groupOfNames)(objectClass=groupOfUniqueNames))"
        dict = self.ldapTree.getNodeDict( self.user, base, groupSchema,
					  filter=filter )
        self.failUnless( dict['base'].mappingDN == 'ou=Groups,o=test' )
        self.failUnless( dict.has_key('ou=Groups,o=test') )

    def test02(self):
        """ 02 testZLDAPTree: getTreeQuery """
        form = { 'targetAttribute': 'manager',
                 'selectDN_manager': 'The Boss',
                 'cn': 'Boss' }
        self.REQUEST.form = form
        query = self.ldapTree.getTreeQuery( self.REQUEST )
        self.failUnless( query == \
            'targetAttribute=manager&cn=Boss&selectDN_manager=The+Boss' )

    def test03(self):
        """ 03 testZLDAPTree: testing orgUnitMemberBrowser """
        request = { 'mappingDN' : 'ou=ST ScI,o=test',
                    'user' : self.user, 
                    'tree-colspan' : 2,
                    'tree-level' : 0,
                    'URL0' : 'http://bogus',
                    'searchURL' : 'http://bogus?query=' }
 
        output = self.ldapTree.orgUnitMemberBrowser( None, request,
                    standard_html_header=(
                        '<TR><TD WIDTH="16" NOWRAP></TD>'
                        '<TD COLSPAN="2" VALIGN="TOP">'),
                    standard_html_footer='</TD></TR>' )
        self.failUnless( index( output, 
                                'href="http://bogus?query=uid%3Dusera"') > 0 )

    def test04(self):
        """ 04 testZLDAPTree: testing orgUnitMemberBrowser by dn"""
        request = { 'mappingDN' : 'ou=ST ScI,o=test',
                    'user' : self.user, 
                    'tree-colspan' : 2,
                    'tree-level' : 0,
                    'URL0' : 'http://bogus',
                    'searchURL' : 'http://bogus?query=' }
 
        schema = self.user.LDAPDirectory.getPersonSchema()
        schema.setPersonsOUAttr('dn')
        output = self.ldapTree.orgUnitMemberBrowser( None, request,
                    standard_html_header=(
                        '<TR><TD WIDTH="16" NOWRAP></TD>'
                        '<TD COLSPAN="2" VALIGN="TOP">'),
                    standard_html_footer='</TD></TR>' )
        self.failUnless( index( output, 
                                'href="http://bogus?query=uid%3Dabogus"') > 0 )

    def test05(self):
        """ 05 testZLDAPTree: testing orgUnitMemberBrowser private"""
        request = { 'mappingDN' : 'ou=ST ScI,o=test',
                    'user' : self.user, 
                    'tree-colspan' : 2,
                    'tree-level' : 0,
                    'URL0' : 'http://bogus',
                    'private' : 1 }
 
        output = self.ldapTree.orgUnitMemberBrowser( None, request,
                    standard_html_header=(
                        '<TR><TD WIDTH="16" NOWRAP></TD>'
                        '<TD COLSPAN="2" VALIGN="TOP">'),
                    standard_html_footer='</TD></TR>' )
        self.failUnless( index( output, 
                 'href="http://bogus?distinguishedName=uid%3Dusera') > 0 )

    def test06(self):
        """ 06 testZLDAPTree: testing orgUnitMemberBrowser private by dn"""
        request = { 'mappingDN' : 'ou=ST ScI,o=test',
                    'user' : self.user, 
                    'tree-colspan' : 2,
                    'tree-level' : 0,
                    'URL0' : 'http://bogus',
                    'private' : 1 }
 
        schema = self.directory.getPersonSchema()
        schema.setPersonsOUAttr('dn')
        output = self.ldapTree.orgUnitMemberBrowser( None, request, 
                    standard_html_header=(
                        '<TR><TD WIDTH="16" NOWRAP></TD>'
                        '<TD COLSPAN="2" VALIGN="TOP">'),
                    standard_html_footer='</TD></TR>' )
        self.failUnless( index( output, 
                 'href="http://bogus?distinguishedName=uid%3Dabogus') > 0 )

    def tearDown(self):
        self.user.unbind()

if __name__ == '__main__':
    
    tests = unittest.makeSuite(testZLDAPTree, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
