"""

    Module: $RCSfile: TestZObject.py,v $
    Author: $Author: payne $
    Date:   $Date: 2003/12/09 02:05:46 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.11 $'[11:-2]

import os, sys
import ZopeEnviron

import Zope
import ZODB
import ZServer
from Acquisition import Implicit
from Globals import DTMLFile
from ZPublisher.Request import Request
from ZPublisher.Response import Response

from string import index
import unittest
from urllib import quote_plus, unquote_plus

from Connection import AnonymousUser
from Connection import AuthenticatedUser
from LDAPExceptions import ALREADY_EXISTS
from LDAPUtilities import listsAreEquivalent
from ManagedRoles import ManagedRoles
from RequestTest import request
from TestDirectory import getTestDirectory
from ZLDAPTree import ZLDAPTree
from ZObject import ZObject
from ZSchema import ZSchema

app = Zope.app()
directory = getTestDirectory()

class testZObject(unittest.TestCase,ZLDAPTree,Implicit):

    abs = os.environ['PWD']+'/../' 
    orgChartForm = DTMLFile(abs+'dtml/orgChart', globals())
    searchForm = DTMLFile(abs+'dtml/search', globals())
    viewForm = DTMLFile(abs+'dtml/view', globals())

    class myZObject(ZObject, ZLDAPTree):
	# Wrapper for ZObject so we can tell it where the form templates are
	# and explicitly subclass mix-ins normally acquired from the
	# STLDAPManager class.
	def __init__(self, user, REQUEST, message=None):
	     ZObject.__init__(self, user, REQUEST, message )

	abs = os.environ['PWD']+'/../' 
        manage_objectFormSelection = DTMLFile(abs+'dtml/objectFormSelection', globals())
        _browseGroupsForm = DTMLFile(abs+'dtml/objectBrowseGroups', globals())
        _browseOrgUnitsForm = DTMLFile(abs+'dtml/objectBrowseOrgUnits', globals())
        _browsePeopleForm = DTMLFile(abs+'dtml/objectBrowsePeople', globals())
        _displayForm = DTMLFile(abs+'dtml/objectDisplay', globals())
        _objectAddForm = DTMLFile(abs+'dtml/objectAdd', globals())
        _objectDelForm = DTMLFile(abs+'dtml/objectDel', globals())
        _objectGroupMembershipForm = DTMLFile(abs+'dtml/objectGroupMembership', globals())
        _objectRenameForm = DTMLFile(abs+'dtml/objectRename', globals())
        _objectDelPersonForm = DTMLFile(abs+'dtml/objectDelPerson', globals())
        _objectRenamePersonForm = DTMLFile(abs+'dtml/objectRenamePerson', globals())
        _renamePersonForm = DTMLFile(abs+'dtml/renamePerson', globals())
        _searchForm = DTMLFile(abs+'dtml/objectSearch', globals(),
                               management_view='People')
        _selectGroupDNForm = DTMLFile(abs+'dtml/selectGroupDN', globals())
        _selectOrgUnitDNForm = DTMLFile(abs+'dtml/selectOrgUnitDN', globals())
        _selectGroupByBrowsingForm = DTMLFile(abs+'dtml/selectGroupByBrowsing',
                               globals(), management_view='Groups')
        _selectOrgUnitByBrowsingForm = DTMLFile(abs+'dtml/selectOrgUnitByBrowsing',
                               globals(), management_view='Org Units')
        _selectPersonByBrowsingForm = DTMLFile(abs+'dtml/selectPersonByBrowsing',
                               globals(), management_view='People')
        _selectPersonDNForm = DTMLFile(abs+'dtml/selectPersonDN', globals())
        _selectPersonOrgUnitDNForm = DTMLFile(abs+'dtml/selectPersonOrgUnitDN',globals())
        _updateJpegPhotoForm = DTMLFile(abs+'dtml/updateJpegPhoto', globals())

    def setUp(self):
	# Set object id so tree states don't change from run to run
        self._p_oid = 'test'
	self = self.__of__(app)
	directory = getTestDirectory()
#	directory.setPersonSearchBase('ou=People,o=test')
#	directory.setOrgUnitBrowseBase('ou=ST ScI,o=test')
        self.user = AuthenticatedUser('testadmin', 'testadmin',
                       directory)
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
        REQUEST.set('objectCategory', 'person')
        zSchema = ZSchema( self.user, REQUEST )
	self.zSchema = zSchema.__of__(self)
        self.zSchema.updateSchemaFromLDAP(self.user)
        self.zObject = ZObject( self.user, REQUEST )

    def test02(self):
        """ 02 TestZObject: testing rename object form """
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set('unitTest', 1 )
	REQUEST.set('objectCategory','group')
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "renameObject"
	REQUEST.form["returnsToFormID"] = "renameObject"
	REQUEST.form["targetAttribute"] = "objectDN"
	REQUEST.form["pointsTo"] = "group"
	REQUEST.form['objectDN'] = ['cn=Test Group,ou=Groups,o=test',]
	REQUEST.form['namingAttributeValue'] = 'Renamed Test Group'
	REQUEST.form['renameObject'] = ' Rename '
	zObject = ZObject( self.user, REQUEST )
        message = zObject.updateDirectoryFromRequest()
        self.failUnless( index( message, 'Renamed object' ) == 0 )

	REQUEST.set('objectDN',['cn=Renamed Test Group,ou=Groups,o=test'])
	REQUEST.set('namingAttributeValue','Test Group')
	self.zObject.request = REQUEST
	message = self.zObject._renameObject()

    def test03(self):
        """ 03 TestZObject: testing group add/remove members forms """
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "groupMembership"
	REQUEST.form["returnsToFormID"] = "groupMembership"
	REQUEST.form["pointsTo"] = "group"
	REQUEST.form['distinguishedName'] = 'cn=Test Group,ou=Groups,o=test'
	REQUEST.form['ids'] = ['uid=usera,ou=People,o=test']
	REQUEST.form['addMembers'] = " Add "
	zObject = ZObject( self.user, REQUEST )
        message = zObject.updateDirectoryFromRequest()
        self.failUnless( index( message, 'Added group members' ) == 0 )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "groupMembership"
	REQUEST.form["returnsToFormID"] = "groupMembership"
	REQUEST.form["pointsTo"] = "group"
	REQUEST.form['distinguishedName'] = 'cn=Test Group,ou=Groups,o=test'
	REQUEST.form['ids'] = ['uid=usera,ou=People,o=test']
	REQUEST.form['areYouSure'] = 1
	REQUEST.form['removeMembers'] = ' Remove '
	zObject = ZObject( self.user, REQUEST )
        message = zObject.updateDirectoryFromRequest()
        self.failUnless( index( message, 'Removed group members' ) == 0 )

    def test04(self):
        """ 04 TestZObject: anonymous user forms """
	# These forms don't actually innstantiate a ZObject.  That is
	# why this class subclasses ZLDAPTree (and Implicit).

	# search for 'user' returns usera and userb
	# 'Username' appears in descriptive text and as a column
	# heading.
        user = AnonymousUser(self.user.getLDAPDirectory())
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",user)
	REQUEST.form["query"] = "user"
	self.REQUEST = REQUEST
	response = self.searchForm(self,REQUEST)
	del self.REQUEST
	self.failUnless(index(response,'Username')>0)
	self.failUnless(index(response,'usera')>0)
	self.failUnless(index(response,'userb')>0)

	# search for 'xanadu' returns no entries
        user = AnonymousUser(self.user.getLDAPDirectory())
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",user)
	REQUEST.form["query"] = "xanadu"
	self.REQUEST = REQUEST
	response = self.searchForm(self,REQUEST)
	del self.REQUEST
	self.failUnless(index(response,'No entries match your query.')>0)

	# default orgChart view should find usera, not userb or abogus
	# and see the DestinationOU object
        user = AnonymousUser(self.user.getLDAPDirectory())
	user.getLDAPDirectory().setPersonSearchBase('ou=People,o=test')
	user.getLDAPDirectory().setOrgUnitBrowseBase('ou=ST ScI,o=test')
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",user)
	REQUEST.set("objectCategory",'person')
	REQUEST.set("URL1","http://myhost/test")
	self.REQUEST = REQUEST
	response = self.orgChartForm(self,REQUEST)
	self.failUnless(index(response,'query=uid%3Dusera')>0)
	self.failUnless(index(response,'query=ou%3DDestinationOU')>0)
	self.failUnlessRaises(ValueError, index, response,'abogus')

	# orgChart using distinguishedName as org unit indicator should
	# find abogus.
        user = AnonymousUser(self.user.getLDAPDirectory())
	dir = user.getLDAPDirectory()
	dir.getPersonSchema().setPersonsOUAttr('dn')
	dir.setPersonSearchBase('ou=People,o=test')
	dir.setOrgUnitBrowseBase('ou=ST ScI,o=test')
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",user)
	REQUEST.set("objectCategory",'person')
	REQUEST.set("URL","http://myhost/test")
	REQUEST.set("URL1","http://myhost")
	self.REQUEST = REQUEST
	response = self.orgChartForm(self,REQUEST)
	del self.REQUEST
	self.failUnless(index(response,'query=uid%3Dabogus')>0)
	self.failUnless(index(response,'query=ou%3DDestinationOU')>0)
	self.failUnlessRaises(ValueError, index, response,'bbogus')

	# orgChart expanded at DestinationOU should find bbogus
        user = AnonymousUser(self.user.getLDAPDirectory())
	dir = user.getLDAPDirectory()
	dir.getPersonSchema().setPersonsOUAttr('dn')
	dir.setPersonSearchBase('ou=People,o=test')
	dir.setOrgUnitBrowseBase('ou=ST ScI,o=test')
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",user)
	REQUEST.set("objectCategory",'person')
	REQUEST.set("URL","http://myhost/test")
	REQUEST.set("URL1","http://myhost")
	REQUEST.form["tree-e"] = "eJyLVk9xD6tKcbS1VddRUM8vtXVJLS7JzEssyczP8w/VAQoEhygEJ3vq5NuWAGXUYwGKChAt"
	REQUEST.cookies["tree-s"] = "eJzTiFZPcQ-rSnG0tVWP1dEEACJ5BB4"
	self.REQUEST = REQUEST
	response = self.orgChartForm(self,REQUEST)
	del self.REQUEST
	self.failUnless(index(response,'query=uid%3Dbbogus')>0)
	self.failUnlessRaises(ValueError, index, response,'abogus')

    def test05(self):
        """ 05 TestZObject: People search form, unprivileged user  """

	# search for 'user' without pressing search button
	# User usera is has no privs and should not see the add/delete
	# person links or the People tab.  The distinguishedName=uid=usera... 
	# appears in the 'My Entry' link, so look for it twice if it is 
	# supposed to be in the search results.  Look for Help link.
        self.user = AuthenticatedUser('usera','usera',self.user.getLDAPDirectory())
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("objectFormID","search")
	REQUEST.set("objectCategory","person")
	REQUEST.set("URL","http://myhost/test")
	REQUEST.set("URL1","http://myhost")
	REQUEST.set("BASEPATH1","")
	zObject = self.myZObject( self.user, REQUEST )
	REQUEST.form["query"] = "user"
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnlessRaises(ValueError, index, response,'objectFormID=addObject')
	self.failUnlessRaises(ValueError, index, response,'objectFormID=delObject')
	try:
	    self.failUnless(index(response,'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')>0)
	    i = index(response,'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')
	    self.failUnless(index(response[i+1:],'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')>0)
	except:
	    self.fail()
	self.failUnless(index(response,'distinguishedName=uid%3Duserb%2Cou%3DPeople%2Co%3Dtest')>0)

	# same search for 'user' but pressing search button
        self.user = AuthenticatedUser('usera','usera',self.user.getLDAPDirectory())
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("objectFormID","search")
	REQUEST.set("objectCategory","person")
	REQUEST.set("URL","http://myhost/test")
	REQUEST.set("URL1","http://myhost")
	REQUEST.set("BASEPATH1","")
	REQUEST.form["query"] = "user"
	REQUEST.form["ldapSearch"] = " Search "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	try:
	    self.failUnless(index(response,'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')>0)
	    i = index(response,'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')
	    self.failUnless(index(response[i+1:],'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')>0)
	except:
	    self.fail()
	self.failUnless(index(response,'distinguishedName=uid%3Duserb%2Cou%3DPeople%2Co%3Dtest')>0)

	# search for bogus string returns no entries message
        self.user = AuthenticatedUser('usera','usera',self.user.getLDAPDirectory())
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("objectFormID","search")
	REQUEST.set("objectCategory","person")
	REQUEST.set("URL","http://myhost/test")
	REQUEST.set("URL1","http://myhost")
	REQUEST.set("BASEPATH1","")
	REQUEST.form["query"] = "xanadu"
	REQUEST.form["ldapSearch"] = " Search "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,'No entries match your query.')>0)
	try:
	    self.failUnless(index(response,'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')>0)
	    i = index(response,'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')
	    self.failUnlessRaises(ValueError, index, response[i+1:],'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')
	except:
	    self.fail()
	self.failUnlessRaises(ValueError, index, response,'distinguishedName=uid%3Duserb%2Cou%3DPeople%2Co%3Dtest')

    def test06(self):
        """ 06 TestZObject: People browse form, unprivileged user  """

	# default org chart view should have links to 'ST ScI' and
	# "Destination OU' and to usera
        self.user = AuthenticatedUser('usera','usera',self.user.getLDAPDirectory())
	dir = self.user.getLDAPDirectory()
	dir.setPersonSearchBase('ou=People,o=test')
	dir.setOrgUnitBrowseBase('ou=ST ScI,o=test')
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("objectFormID","orgChart")
	REQUEST.set("objectCategory","person")
	REQUEST.set("URL","http://myhost/test")
	REQUEST.set("URL1","http://myhost")
	REQUEST.set("BASEPATH1","")
	zObject = self.myZObject( self.user, REQUEST )
        zObject._p_oid = 'test'
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,'ST ScI')>0)
	self.failUnless(index(response,'#ou=DestinationOU,ou=ST ScI,o=test')>0)
	try:
	    self.failUnless(index(response,'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')>0)
	    i = index(response,'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')
	    self.failUnless(index(response[i+1:],'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')>0)
	except:
	    self.fail()

	# open org chart view should have links to 'ST ScI' and
	# "Destination OU' but not usera
        self.user = AuthenticatedUser('usera','usera',self.user.getLDAPDirectory())
	dir = self.user.getLDAPDirectory()
	dir.setPersonSearchBase('ou=People,o=test')
	dir.setOrgUnitBrowseBase('ou=ST ScI,o=test')
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("objectFormID","orgChart")
	REQUEST.set("objectCategory","person")
	REQUEST.set("URL","http://myhost/test")
	REQUEST.set("URL1","http://myhost")
	REQUEST.set("BASEPATH1","")
	REQUEST.form["tree-e"] = "eJyLVk9xD6tKcbS1VddRUM8vtXVJLS7JzEssyczP8w/VAQoEhygEJ3vq5NuWAGXUYwGKChAt"
	REQUEST.cookies["tree-s"] = "eJzTiFZPcQ-rSnG0tVWP1dEEACJ5BB4"
	zObject = self.myZObject( self.user, REQUEST )
        zObject._p_oid = 'test'
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	i = index(response,'ST ScI')
	self.failUnless(index(response[i:],'tree-c')>0)
	self.failUnless(index(response[i:],'#ou=DestinationOU,ou=ST ScI,o=test')>0)
	try:
	    self.failUnless(index(response,'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')>0)
	    i = index(response,'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')
	    self.failUnlessRaises(ValueError,index,response[i+1:],'distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')
	except:
	    self.fail()

	# Pressing the Cancel button in the expanded state should 
	# expire the tree-s cookie, and then do a redirect, which
	# we can't really test.
        self.user = AuthenticatedUser('usera','usera',self.user.getLDAPDirectory())
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("objectFormID","orgChart")
	REQUEST.set("objectCategory","person")
	REQUEST.set("URL","http://myhost/test")
	REQUEST.set("URL1","http://myhost")
	REQUEST.set("BASEPATH1","")
	REQUEST.cookies["tree-s"] = "eJyLjlZPcQ-rSnG0tVXXUYiOVs8vtXVJLS7JzEssyczP8w/VAQoEhygEJ3vq5NuWAGVAymJBAAA9/hNZ"
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["treeCancel"] = " Cancel "
	zObject = self.myZObject( self.user, REQUEST )
        zObject._p_oid = 'test'
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	zObject.showObjectForm()
	response = str(zObject.request.RESPONSE)
	self.failUnless(index(response,'Set-Cookie: tree-s')>0)
	i = index(response,'Set-Cookie: tree-s')
	self.failUnless(index(response[i+1:],'Expires=')>0)

    def test07(self):
        """ 07 TestZObject: People add form, privileged user """

	# Pressing Browse button returns a selection form.
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("objectCategory","person")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "addObject"
	REQUEST.form["returnsToFormID"] = "addObject"
	REQUEST.form["targetAttribute"] = "parentDN"
	REQUEST.form["pointsTo"] = "personOrgUnit"
	REQUEST.form["selectDN_parentDN"] = " Browse "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,'Select An Organizational Unit for a Person')>0)

	# Return from selecting a parent
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("objectCategory","person")
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["targetAttribute"] = "parentDN"
	REQUEST.form["pointsTo"] = "personOrgUnit"
	REQUEST.form["updateObject"] = 1
	REQUEST.form["returnsToFormID"] = "addObject"
	REQUEST.form["objectFormID"] = "addObject"
	REQUEST.form["parentDN"] = "ou=People,o=test"
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,'?distinguishedName=ou%3DPeople%2Co%3Dtest')>0)
	self.failUnless(index(response,'New Username:')>0)

	# Cancel out from re-selecting a parent
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("objectCategory","person")
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["targetAttribute"] = "parentDN"
	REQUEST.form["namingAttributeValue"] = "cbogus"
	REQUEST.form["pointsTo"] = "personOrgUnit"
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["returnsToFormID"] = "addObject"
	REQUEST.form["parentDN"] = "ou=People,o=test"
	REQUEST.form["objectFormID"] = "addObject"
	REQUEST.form["treeCancel"] = " Cancel "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,'?distinguishedName=ou%3DPeople%2Co%3Dtest')>0)
	self.failUnless(index(response,'value="cbogus"')>0)

	# Submit a namingAttribute value, w/ parentDN, pressing
	# the Add button.  Can't test the javascript that makes this
	# work without pressing the button.
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.set("objectCategory","person")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "addObject"
	REQUEST.form["returnsToFormID"] = "addObject"
	REQUEST.form["targetAttribute"] = "parentDN"
	REQUEST.form["pointsTo"] = "personOrgUnit"
	REQUEST.form["parentDN"] = quote_plus("ou=People,o=test")
	REQUEST.form["namingAttributeValue"] = "cbogus"
	REQUEST.form["addObject"] = " Add "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	response = str(zObject.request.RESPONSE)
	self.failUnless(index(response,'manage_tabs_message=Added+object+%27uid%3Dcbogus%2Cou%3DPeople%2Co%3Dtest')>0)

	# Should fail if repeated
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.set("objectCategory","person")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "addObject"
	REQUEST.form["returnsToFormID"] = "addObject"
	REQUEST.form["targetAttribute"] = "parentDN"
	REQUEST.form["pointsTo"] = "personOrgUnit"
	REQUEST.form["parentDN"] = quote_plus("ou=People,o=test")
	REQUEST.form["namingAttributeValue"] = "cbogus"
	REQUEST.form["addObject"] = " Add "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,'already exists!')>0)

	# Cancel out at the add stage
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.set("objectCategory","person")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "addObject"
	REQUEST.form["returnsToFormID"] = "addObject"
	REQUEST.form["targetAttribute"] = "parentDN"
	REQUEST.form["pointsTo"] = "personOrgUnit"
	REQUEST.form["parentDN"] = quote_plus("ou=People,o=test")
	REQUEST.form["namingAttributeValue"] = "cbogus"
	REQUEST.form["cancel"] = " Cancel "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	response = str(zObject.request.RESPONSE)
	self.failUnless(index(response,'Moved Temporarily\n')>0)
	self.failUnless(index(response,'?objectFormID=addObject\n')>0)

	# Clean up.
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'person')
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "delObject"
	REQUEST.form["returnsToFormID"] = "delObject"
	REQUEST.form["pointsTo"] = "person"
        REQUEST.form['ids'] = [ "uid=cbogus,ou=People,o=test" ]
        REQUEST.form['areYouSure'] =  1
	REQUEST.form['delObject'] = " Yes, I'm sure "
	zObject = ZObject( self.user, REQUEST )
        message = zObject.updateDirectoryFromRequest()
	self.failUnless(index(message,"Objects deleted.")==0)

    def test08(self):
        """ 08 TestZObject: People del form, privileged user """

	# Search for objects.
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'person')
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "delObject"
	REQUEST.form["returnsToFormID"] = "delObject"
	REQUEST.form["pointsTo"] = "person"
	REQUEST.form["query"] = "user"
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,'?distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')>0)
	self.failUnless(index(response,'?distinguishedName=uid%3Duserb%2Cou%3DPeople%2Co%3Dtest')>0)

	# Browse for objects.
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'person')
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "delObject"
	REQUEST.form["returnsToFormID"] = "delObject"
	REQUEST.form["pointsTo"] = "person"
	REQUEST.form["query"] = "user"
	REQUEST.form["selectObjectByBrowsing"] = " Browse "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,'Select A Person By Browsing')>0)

	# Cancel out of browsing for objects should preserve query
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'person')
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "delObject"
	REQUEST.form["returnsToFormID"] = "delObject"
	REQUEST.form["pointsTo"] = "person"
	REQUEST.form["query"] = "user"
	REQUEST.form["treeCancel"] = " Cancel "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,'?distinguishedName=uid%3Dusera%2Cou%3DPeople%2Co%3Dtest')>0)
	self.failUnless(index(response,'?distinguishedName=uid%3Duserb%2Cou%3DPeople%2Co%3Dtest')>0)

	# Cancel out of searching should NOT preserve query
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'person')
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "delObject"
	REQUEST.form["returnsToFormID"] = "delObject"
	REQUEST.form["pointsTo"] = "person"
	REQUEST.form["query"] = "user"
	REQUEST.form["cancel"] = " Cancel "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	response = str(zObject.request.RESPONSE)
	self.failUnless(index(response,'Location:')>0)
	self.failUnless(index(response,'?objectFormID=delObject\n')>0)

	# An object to delete
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.set("objectCategory","person")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "addObject"
	REQUEST.form["returnsToFormID"] = "addObject"
	REQUEST.form["targetAttribute"] = "parentDN"
	REQUEST.form["pointsTo"] = "personOrgUnit"
	REQUEST.form["parentDN"] = quote_plus("ou=People,o=test")
	REQUEST.form["namingAttributeValue"] = "cbogus"
	REQUEST.form["addObject"] = " Add "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	response = str(zObject.request.RESPONSE)
	self.failUnless(index(response,'manage_tabs_message=Added+object+%27uid%3Dcbogus%2Cou%3DPeople%2Co%3Dtest')>0)

	# Delete
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'person')
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "delObject"
	REQUEST.form["returnsToFormID"] = "delObject"
	REQUEST.form["pointsTo"] = "person"
	REQUEST.form["query"] = "bogus"
        REQUEST.form['ids'] = [ "uid=cbogus,ou=People,o=test" ]
        REQUEST.form['areYouSure'] =  1
	REQUEST.form['delObject'] = " Yes, I'm sure "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,"Objects deleted.")>0)

    def test09(self):
        """ 09 TestZObject: Group membership form """

	# Go to Select A Group form
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'group')
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "groupMembership"
	REQUEST.form["returnsToFormID"] = "groupMembership"
	REQUEST.form["pointsTo"] = "group"
	REQUEST.form["selectDN_distinguishedName"] = " Browse for Group "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,"Select A Group")>0)

	# Search for members
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'group')
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "groupMembership"
	REQUEST.form["returnsToFormID"] = "groupMembership"
	REQUEST.form["pointsTo"] = "group"
	REQUEST.form["distinguishedName"] = \
		"cn=Divisional Administrators,ou=Groups,o=test"
	REQUEST.form["query"] = "user"
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,"""<input type="checkbox" name="ids:list"
        value="uid=usera,ou=People,o=test">""")>0)
	self.failUnless(index(response,"""<input type="checkbox" name="ids:list"
        value="uid=userb,ou=People,o=test">""")>0)

	# Go to browse to members form
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'group')
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "groupMembership"
	REQUEST.form["returnsToFormID"] = "groupMembership"
	REQUEST.form["pointsTo"] = "group"
	REQUEST.form["distinguishedName"] = \
		"cn=Divisional Administrators,ou=Groups,o=test"
	REQUEST.form["browseObjectCategory"] = "person"
	REQUEST.form["selectObjectByBrowsing"] = " Browse "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,"Select A Person By Browsing")>0)

	# Cancel search for members
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'group')
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "groupMembership"
	REQUEST.form["returnsToFormID"] = "groupMembership"
	REQUEST.form["pointsTo"] = "group"
	REQUEST.form["distinguishedName"] = \
		"cn=Divisional Administrators,ou=Groups,o=test"
	REQUEST.form["query"] = "user"
	REQUEST.form["cancel"] = " Cancel "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	response = str(zObject.request.RESPONSE)
	self.failUnless(index(response,"Location:")>0)
	self.failUnless(index(response,"distinguishedName=cn%3DDivisional+Administrators%2Cou%3DGroups%2Co%3Dtest")>0)
	self.failUnlessRaises(ValueError, index, response, "query=user")

	# Add members
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'group')
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "groupMembership"
	REQUEST.form["returnsToFormID"] = "groupMembership"
	REQUEST.form["pointsTo"] = "group"
	REQUEST.form["distinguishedName"] = \
		"cn=Divisional Administrators,ou=Groups,o=test"
	REQUEST.form["query"] = "user"
	REQUEST.form["ids"] = [ "uid=usera,ou=People,o=test",
				"uid=userb,ou=People,o=test" ]
	REQUEST.form["addMembers"] = " Add "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,"Added group members")>0)
	i = index(response,"Current Membership:")
	self.failUnless(index(response,"distinguishedName=cn%3DDivisional%20Administrators%2Cou%3DGroups%2Co%3Dtest")>0)
	self.failUnlessRaises(ValueError, index, response[:i],
		"""<input type="checkbox" name="ids:list"
        value="uid=usera,ou=People,o=test">""")
	self.failUnlessRaises(ValueError, index, response[:i],
		"""<input type="checkbox" name="ids:list"
        value="uid=usera,ou=People,o=test">""")
	self.failUnless(index(response[i:],
	"""<input type="checkbox" name="ids:list"
        value="uid=usera,ou=People,o=test">""")>0)
	self.failUnless(index(response[i:],
	"""<input type="checkbox" name="ids:list"
        value="uid=userb,ou=People,o=test">""")>0)

	# Remove members
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("user",self.user)
	REQUEST.set('objectCategory', 'group')
	REQUEST.set("URL1","http://myhost/test")
	REQUEST.form["updateObject"] = "1"
	REQUEST.form["objectFormID"] = "groupMembership"
	REQUEST.form["returnsToFormID"] = "groupMembership"
	REQUEST.form["pointsTo"] = "group"
	REQUEST.form["distinguishedName"] = \
		"cn=Divisional Administrators,ou=Groups,o=test"
	REQUEST.form["query"] = "user"
	REQUEST.form["ids"] = [ "uid=usera,ou=People,o=test",
				"uid=userb,ou=People,o=test" ]
	REQUEST.form["areYouSure"] = 1
	REQUEST.form["removeMembers"] = " Remove "
	zObject = self.myZObject( self.user, REQUEST )
	zObject.REQUEST = REQUEST
	zObject = zObject.__of__(app)
	response = zObject.showObjectForm()
	self.failUnless(index(response,"Removed group members")>0)
	i = index(response,"Current Membership:")
	self.failUnless(index(response,"distinguishedName=cn%3DDivisional%20Administrators%2Cou%3DGroups%2Co%3Dtest")>0)
	self.failUnless(index(response[:i],
	"""<input type="checkbox" name="ids:list"
        value="uid=usera,ou=People,o=test">""")>0)
	self.failUnless(index(response[:i],
	"""<input type="checkbox" name="ids:list"
        value="uid=userb,ou=People,o=test">""")>0)
	self.failUnlessRaises(ValueError, index, response[i:],
		"""<input type="checkbox" name="ids:list"
        value="uid=usera,ou=People,o=test">""")
	self.failUnlessRaises(ValueError, index, response[i:],
		"""<input type="checkbox" name="ids:list"
        value="uid=usera,ou=People,o=test">""")

    def tearDown(self):
        self.user.unbind()

if __name__ == '__main__':

    tests = unittest.makeSuite(testZObject, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
