"""

    Module: $RCSfile: TestZReports.py,v $
    Author: $Author: payne $
    Date:   $Date: 2003/12/15 14:55:19 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.2 $'[11:-2]

import os, sys
import ZopeEnviron

import Zope
import ZODB
import ZServer
from AccessControl import ClassSecurityInfo, getSecurityManager
from AccessControl.Role import RoleManager
from Acquisition import Implicit
from Globals import DTMLFile, InitializeClass
from OFS.Folder import manage_addFolder, Folder
from ZPublisher.Request import Request
from ZPublisher.Response import Response

from copy import deepcopy
from string import index, split
import unittest

from Connection import AuthenticatedUser
from LDAPExceptions import REQUIRED_ATTRIBUTE, NO_SUCH_ATTRIBUTE
from LDAPUtilities import listsAreEquivalent
from ManagedRoles import ManagedRoles
from ManagedSchema import ManagedSchema
from SchemaDefaults_test import *
from STLDAPManager import STLDAPManager
from TestDirectory import *
from ZReports import ZReports
from ZRoles import ZRoles
from ZSchema import ZSchema

app = Zope.app()

class testZReports(unittest.TestCase,Folder):

    managePeoplePermission = STLDAPManager.managePeoplePermission
    manageGroupPermission = STLDAPManager.manageGroupPermission
    manageOrgUnitPermission = STLDAPManager.manageOrgUnitPermission
    manageSchemaPermission = STLDAPManager.manageSchemaPermission

    __ac_roles__ = ('Internal')

    __ac_permissions__ = (
	    (managePeoplePermission,
	      ()),
	    (manageGroupPermission,
	      ()),
	    (manageOrgUnitPermission,
	      ()),
	    (manageSchemaPermission,
	      ()),
	    )

    security = ClassSecurityInfo()
    security.setPermissionDefault(
	managePeoplePermission,
	('Manager', 'Divisional Administrator'),
	)
    security.setPermissionDefault(
	manageGroupPermission,
	('Manager', 'Divisional Administrator'),
	)
    security.setPermissionDefault(
	manageOrgUnitPermission,
	('Manager', 'Divisional Administrator'),
	)
    security.declareProtected(
	managePeoplePermission,'setUp')
    security.declareProtected(
	manageGroupPermission,'dummy1')
    security.declareProtected(
	manageOrgUnitPermission,'dummy2')

    def dummy1(self):
	pass
    def dummy2(self):
	pass

    class myZReports(ZReports):
	# Wrapper for ZReports so we can tell it where the form templates are
	# and explicitly subclass mix-ins normally acquired from the
	# STLDAPManager class.
	def __init__(self, user, REQUEST ):
	    ZReports.__init__(self, user, REQUEST )

	abs = os.environ['PWD']+'/../' 
        manage_reportFormSelection = DTMLFile(abs+'dtml/reportFormSelection', 
					globals())
	getSchemaReport = DTMLFile(abs+'dtml/schemaReport', globals())
	getSchemaAttrReport = DTMLFile(abs+'dtml/schemaAttrReport', globals())
	_reportOpenLDAPForm = DTMLFile(abs+'dtml/reportOpenLDAP',globals())
	_reportIPlanetForm = DTMLFile(abs+'dtml/reportIPlanet',globals())   
	_reportSchemaDefaultsForm = DTMLFile(abs+'dtml/reportSchemaDefaults',
                                          globals())
	_reportSchemaDocForm = DTMLFile(abs+'dtml/reportSchemaDoc', globals())

    def setUp(self):
        get_transaction().begin()
	self = self.__of__(app)
	self.__name__ = 'test'
	self.manage_addFolder('testFolder')

        directory = getTestDirectory()
        self.user = AuthenticatedUser('testadmin', 'testadmin', directory)
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
        zRoles = ZRoles( self.user, REQUEST )
        self.testFolder.zRoles = zRoles.__of__( self)

	REQUEST.set('objectCategory','person')
        zSchema = ZSchema( self.user, REQUEST )
	self.testFolder.zSchema = zSchema.__of__(self)
	self.testFolder.zSchema._assignRolesToPermission()

	REQUEST.set('objectCategory','orgUnit')
        zSchema = ZSchema( self.user, REQUEST )
	self.testFolder.zSchema = zSchema.__of__(self)
	self.testFolder.zSchema._assignRolesToPermission()

	REQUEST.set('objectCategory','group')
        zSchema = ZSchema( self.user, REQUEST )
	self.testFolder.zSchema = zSchema.__of__(self)
	self.testFolder.zSchema._assignRolesToPermission()

    def test01(self):
        """ 01 TestZReports: testing Schema doc form """

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("URL1","http://myhost")
	REQUEST.form["reportsFormID"] = "reportSchemaDoc"
	REQUEST.form["securityFormat"] = "SchemaDoc"
        zReports = self.myZReports( self.user, REQUEST )
	zReports.REQUEST = REQUEST
	zReports.PARENTS = [self.testFolder,]
        self.testFolder.zReports = zReports.__of__(self.testFolder)
	self.testFolder = self.testFolder.__of__(self)
	self = self.__of__(app)
        response = self.testFolder.zReports.showReportsForm()
	self.failUnless(index(response,"""<ul>
  <li>Divisional Administrators:
   LDAP Manage Org Units.
  <li>System Administrators:
   LDAP Manage Groups,&nbsp;LDAP Manage Org Units,&nbsp;LDAP Manage People.
</ul>
""")>0)

	self.failUnless(index(response,"""<a name="personSchema"><h2>Person Schema</h2></a>



<p class="form-text">
The roles that have access to the "Add" and "Delete" object
links for this objectCategory,
selected under the "<a
href="http://myhost/manage_schema?schemaFormID=schemaControllers">Add/Delete
by Role</a>" screen under the Schema tab, are:
<ul>
<li>System Administrators
</ul>
</p>

""")>0)

	# displayName report, minus the first part, which differs
	# between OpenLDAP and iPlanet (where it is single-valued).

	self.failUnless(index(response,"""
  
  <tr>
    <td class="form-label" nowrap>
      Friendly Name:
    </td>
    <td width="16">&nbsp;</td>
    <td align="left" width="100%" class="form-text">
       Display Name
    </td>
  </tr>

  <tr>
    <td valign="top" class="form-label" nowrap>
      Description:
    </td>
    <td width="16">&nbsp;</td>
    <td align="left" width="100%" class="form-text">
      
    </td>
  </tr>

  <tr>
    <td valign="top" class="form-label" nowrap>
      Syntax:
    </td>
    <td width="16">&nbsp;</td>
    <td align="left" width="100%" class="form-text">
      DirectoryString
    </td>
  </tr>

  <tr>
    <td valign="top" class="form-label" nowrap>
      Readable by Roles:
    </td>
    <td width="16">&nbsp;</td>
    <td align="left" width="100%" class="form-text">
          Self,&nbsp;          Anonymous User,&nbsp;          Authenticated User,&nbsp;          Direct Manager,&nbsp;          Divisional Administrators,&nbsp;          System Administrators        </td>
  </tr>

  <tr>
    <td valign="top" class="form-label" nowrap>
      Writeable by Roles:
    </td>
    <td width="16">&nbsp;</td>
    <td align="left" width="100%" class="form-text">
          Self,&nbsp;          Divisional Administrators,&nbsp;          System Administrators        </td>
  </tr>

  
  
</table>
""")>0)

    def test02(self):
        """ 02 TestZReports: testing OpenLDAP security  """

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["reportsFormID"] = "reportOpenLDAP"
	REQUEST.form["securityFormat"] = "OpenLDAP"
        zReports = self.myZReports( self.user, REQUEST )
	zReports.REQUEST = REQUEST
        self.testFolder.zReports = zReports.__of__(self.testFolder)
	self = self.__of__(app)
        response = self.testFolder.zReports.showReportsForm()

	# Check that the naming attribute gets write access by the 
	# object controllers.
	self.failUnless(index(response,"""
# Start of LDAP Manager generated configuration

access to dn="" by * read

access to dn="cn=subschema" by * read

access to dn.subtree="o=test"
          filter=(objectclass=person)
          attrs=children,entry,uid
          by group/groupOfUniqueNames/uniqueMember="cn=System Administrators,ou=Groups,o=test" write
          by * read

""")==0)

	# Check that roomNumber got anonymous read access because
	# it has Zope role read access.
	self.failUnless(index(response,"""
access to dn.subtree="o=test"
          filter=(objectclass=person)
          attrs=roomNumber
          by self write
          by group/groupOfUniqueNames/uniqueMember="cn=Divisional Administrators,ou=Groups,o=test" write
          by group/groupOfUniqueNames/uniqueMember="cn=System Administrators,ou=Groups,o=test" write
          by dnattr=manager read
          by users read
          by anonymous read

""")>0)

	self.failUnless(index(response,"""
access to dn.subtree="o=test"
          filter=(objectclass=organizationalunit)
          attrs=children,entry,ou
          by group/groupOfUniqueNames/uniqueMember="cn=Divisional Administrators,ou=Groups,o=test" write
          by group/groupOfUniqueNames/uniqueMember="cn=System Administrators,ou=Groups,o=test" write
          by * read

""") > 0)

	self.failUnless(index(response,"""
access to dn.subtree="o=test"
          filter=(objectclass=organizationalunit)
          attrs=businessCategory,description,facsimileTelephoneNumber,l,physicalDeliveryOfficeName,postOfficeBox,postalAddress,postalCode,searchGuide,seeAlso,st,street,telephoneNumber
          by group/groupOfUniqueNames/uniqueMember="cn=Divisional Administrators,ou=Groups,o=test" write
          by dnattr=manager read
          by group/groupOfUniqueNames/uniqueMember="cn=System Administrators,ou=Groups,o=test" read
          by users read
          by anonymous read

""") > 0)

	self.failUnless(index(response,"""
access to dn.subtree="o=test"
          filter=(|(objectclass=groupofnames)(objectclass=groupofuniquenames))
          attrs=children,entry,cn
          by group/groupOfUniqueNames/uniqueMember="cn=System Administrators,ou=Groups,o=test" write
          by * read

""") > 0)

	self.failUnless(index(response,"""
access to dn.subtree="o=test"
          filter=(|(objectclass=groupofnames)(objectclass=groupofuniquenames))
          attrs=o
          by group/groupOfUniqueNames/uniqueMember="cn=System Administrators,ou=Groups,o=test" write
          by self read
          by users read

""") > 0)

    def test03(self):
        """ 03 TestZReports: testing IPlanet security """

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["reportsFormID"] = "reportIPlanet"
	REQUEST.form["securityFormat"] = "IPlanet"
        zReports = self.myZReports( self.user, REQUEST )
	zReports.REQUEST = REQUEST
        self.testFolder.zReports = zReports.__of__( self.testFolder )
	self = self.__of__(app)
        response = self.testFolder.zReports.showReportsForm()

	# Check for saved iPlanetACIs
	self.failUnless(index(response,"""dn: o=test
changetype: modify
delete: aci
-
add: aci
aci:
 (targetattr = "*")(version 3.0;acl "Configuration Adminstrator"; allow (all)userdn = "ldap:///uid=admin, ou=Administrators, ou=TopologyManagement, o=NetscapeRoot";)
-""")==0)

	# Check that roomNumber is promoted to anonymous read since
	# read access from a Zope role (Internal) is specified.
	self.failUnless(index(response,"""add: aci
aci:
 (target = "ldap:///o=test")
 (targetattr = "businessCategory || cn || departmentNumber || description || displayName || facsimileTelephoneNumber || givenName || initials || l || mail || objectClass || ou || physicalDeliveryOfficeName || postOfficeBox || postalAddress || postalCode || roomNumber || seeAlso || sn || st || street || telephoneNumber || title || uid")
 (targetfilter=(objectclass=person))
 (version 3.0;
 acl "ANON can read from persons";
 allow (read,compare,search)
 (userdn = "ldap:///anyone");)
-
""")>0)

	# Basic write check.
	self.failUnless(index(response,"""add: aci
aci:
 (target = "ldap:///o=test")
 (targetattr = "carLicense || cn || description || displayName || facsimileTelephoneNumber || givenName || homePhone || homePostalAddress || initials || l || mobile || ou || pager || postOfficeBox || roomNumber || seeAlso || sn || userPassword")
 (targetfilter=(objectclass=person))
 (version 3.0;
 acl "SELF can write to persons";
 allow (read,compare,search,write)
 (userdn = "ldap:///self");)
-
""")>0)

    def test04(self):
        """ 04 TestZReports: testing iPlanetACIs"""

	ACIs = """-
Not really an aci.
Just a test.
"""
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("URL1","http://myhost")
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["reportsFormID"] = "reportIPlanet"
	REQUEST.form["iPlanetACIs"] = split(ACIs,'\n')
	REQUEST.form["saveIPlanetACIs"] = " Save "
        zReports = self.myZReports( self.user, REQUEST )
	zReports.REQUEST = REQUEST
        self.testFolder.zReports = zReports.__of__( self.testFolder )
	self = self.__of__(app)
        response = self.testFolder.zReports.showReportsForm()
	self.failUnless(index(response,"Updated saved iPlanet ACIs.")>0)

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["reportsFormID"] = "reportIPlanet"
	REQUEST.form["securityFormat"] = "IPlanet"
        zReports = self.myZReports( self.user, REQUEST )
	zReports.REQUEST = REQUEST
        self.testFolder.zReports = zReports.__of__( self.testFolder )
        response = self.testFolder.zReports.showReportsForm()
	self.failUnless(index(response,ACIs)>0)

    def test05(self):
        """ 05 TestZReports: testing SchemaDefaults report """

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["reportsFormID"] = "reportSchemaDefaults"
	REQUEST.form["securityFormat"] = "SchemaDefaults"
        zReports = self.myZReports( self.user, REQUEST )
	zReports.REQUEST = REQUEST
        self.testFolder.zReports = zReports.__of__(self.testFolder)
	self.testFolder = self.testFolder.__of__(self)
	self = self.__of__(app)
        response = self.testFolder.zReports.showReportsForm()
	self.failUnless(index(response,"""
#   personsOUAttr is the key for determing a person's org unit: 'dn' if
#   you look at a person's dn, or the name of the person attribute that
#   contains the person's org unit.
#   groupObjectClass can be either 'groupOfNames' or 'groupOfUniqueNames'.

defaultPersonsOUAttr = 'ou'
defaultGroupObjectClass = 'groupOfUniqueNames'

#   Controllers are the roles that can add/delete entries of a given
#   object category.

defaultGroupControllers = ['System Administrators']
defaultOrgUnitControllers = ['Divisional Administrators', 'System Administrators']
defaultUserControllers = ['System Administrators']
""")>0)

	self.failUnless(index(response,"""
        'displayName' : {
                'friendlyName' : 'Display Name',
                'readers' : ['SELF', 'ANON', 'AUTH', 'MNGR', 'Divisional Administrators', 'System Administrators'],
                'writers' : ['SELF', 'Divisional Administrators', 'System Administrators'],
                'displayCategories' : ['General'], },
""")>0)

    def tearDown(self):
        self.user.unbind()

InitializeClass(testZReports)

if __name__ == '__main__':

    tests = unittest.makeSuite(testZReports, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
