"""

    Module: $RCSfile: TestZRoles.py,v $
    Author: $Author: payne $
    Date:   $Date: 2003/12/07 04:13:54 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.    

"""
__version__ = '$Revision: 1.21 $'[11:-2]


import os, sys
import ZopeEnviron

import Zope
import ZODB
import ZServer
import Globals
from AccessControl import ClassSecurityInfo, getSecurityManager
from AccessControl.Role import RoleManager
from Acquisition import Implicit
from ZPublisher.Request import Request
from ZPublisher.Response import Response

from copy import deepcopy
from string import index
import unittest

from Connection import AuthenticatedUser
from LDAPExceptions import *
from LDAPUtilities import *
from ManagedRoles import ManagedRoles
from ManagedSchema import ManagedSchema
from RequestTest import request
from SchemaDefaults_test import *
from STLDAPManager import STLDAPManager
from TestDirectory import getTestDirectory
from ZRoles import ZRoles
from ZSchema import ZSchema

class testZRoles(unittest.TestCase,RoleManager,Implicit):

    __ac_roles__ = ('Divisional Administrator', 'System Administrator' )

    __ac_permissions__ = (
	    ('STLDAPManager.managePeoplePermission',
	      ()),
	    ('STLDAPManager.manageGroupPermission',
	      ()),
	    ('STLDAPManager.manageOrgUnitPermission',
	      ()),
	    )

    security = ClassSecurityInfo()
    security.setPermissionDefault(
	STLDAPManager.managePeoplePermission,
	('Manager', 'Divisional Administrator'),
	)
    security.declareProtected(
	STLDAPManager.managePeoplePermission,'setUp')

    def setUp(self):
        get_transaction().begin()
	self = self.__of__( Zope.app() )

        directory = getTestDirectory()
        self.user = AuthenticatedUser('testadmin', 'testadmin',
                       directory)
        zSchema = ZSchema( self.user, {'objectCategory':'person'} )
	self.zSchema = zSchema.__of__(self)
        self.zSchema.updateSchemaFromLDAP(self.user)
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)

    def test01(self):
        """ 01 testZRoles: testing Roles form"""

	# First, selecting nothing should do nothing.

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["changeRoles"] = 1
	REQUEST.form["rolesFormID"] = "rolesSelect"
	REQUEST.form['ids'] = []
	REQUEST.form["deleteConfiguredRoles"] = " Delete "
        zRoles = ZRoles( self.user, REQUEST )
        self.zRoles = zRoles.__of__( self )
	self.zSchema._assignRolesToPermission()
        message = self.zRoles.updateRolesFromRequest()
        self.failUnless( message == "" )
	roles = self.zRoles.getManagedRoles()
        self.failUnless( 'System Administrators' in \
			 roles.getConfiguredRoleNames() )

	# System Administrators should be valid role
	self.failUnless( 'System Administrators' in self.valid_roles())

	# Delete 'System Administrators'

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["changeRoles"] = 1
	REQUEST.form["rolesFormID"] = "rolesSelect"
	REQUEST.form['ids'] = ['System Administrators',]
	REQUEST.form["deleteConfiguredRoles"] = " Delete "
        zRoles = ZRoles( self.user, REQUEST )
        self.zRoles = zRoles.__of__( self )
        message = self.zRoles.updateRolesFromRequest()
        self.failUnless(index(message,"Deleted role(s).")==0 )
	roles = self.zRoles.getManagedRoles()
        self.failIf( 'System Administrators' in \
		     roles.getConfiguredRoleNames() )

	# System Administrators should not be valid role
	self.failIf( 'System Administrators' in self.valid_roles())

	# Add nothing should do nothing

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["changeRoles"] = 1
	REQUEST.form["rolesFormID"] = "rolesSelect"
	REQUEST.form['newRoles'] = []
	REQUEST.form['addRoles'] = " Add "
        zRoles = ZRoles( self.user, REQUEST )
        self.zRoles = zRoles.__of__( self )
        message = self.zRoles.updateRolesFromRequest()
        self.failUnless( message == "" )
	roles = self.zRoles.getManagedRoles()
        self.failIf( 'System Administrators' in \
			 roles.getConfiguredRoleNames() )

	# Now add for real

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["changeRoles"] = 1
	REQUEST.form["rolesFormID"] = "rolesSelect"
	REQUEST.form['newRoles'] = ['System Administrators',]
	REQUEST.form['addRoles'] = " Add "
        zRoles = ZRoles( self.user, REQUEST )
        self.zRoles = zRoles.__of__( self )
        message = self.zRoles.updateRolesFromRequest()
        self.failUnless(index(message,"Added roles.")==0 )
	roles = self.zRoles.getManagedRoles()
        self.failUnless( 'System Administrators' in \
			 roles.getConfiguredRoleNames() )

	# System Administrators should be valid role
	self.failUnless( 'System Administrators' in self.valid_roles())

	# Rename nothing

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["changeRoles"] = 1
	REQUEST.form["rolesFormID"] = "rolesSelect"
	REQUEST.form['ids'] = []
	REQUEST.form['renamePermanentRoles'] = " Rename "
        zRoles = ZRoles( self.user, REQUEST )
        self.zRoles = zRoles.__of__( self )
        message = self.zRoles.updateRolesFromRequest()
        self.failUnless( message == "" )

	# Renaming should go to the renaming form

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["changeRoles"] = 1
	REQUEST.form["rolesFormID"] = "rolesSelect"
	REQUEST.form['ids'] = ['MNGR']
	REQUEST.form['renamePermanentRoles'] = " Rename "
        zRoles = ZRoles( self.user, REQUEST )
        self.zRoles = zRoles.__of__( self )
        message = self.zRoles.updateRolesFromRequest()
	request = self.zRoles.request
	self.failUnless(request.get('rolesFormID')=='rolesRename')
	self.failIf(request.has_key('updateSchema'))

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["changeRoles"] = 1
	REQUEST.form["rolesFormID"] = "rolesSelect"
        REQUEST.form['haveNewNames'] = 1
	REQUEST.form['renamePermanentRoles'] = " Rename "
        REQUEST.form['ids'] = ['SELF', 'ANON', 'MNGR']
        REQUEST.form['new_ids'] = ['Me', '', 'Direct Manager']
#        roles = ['Me', 'Anonymous User', 'Authenticated User', 
#                 'Direct Manager', 'Allow access to home info']
        roles = ['Me', 'Anonymous User', 'Authenticated User', 
                 'Direct Manager', ]
        zRoles = ZRoles( self.user, REQUEST )
        self.zRoles = zRoles.__of__( self )
        message = self.zRoles.updateRolesFromRequest()
        self.failUnless(index(message,"Renamed roles categories.") == 0)
	renamedRoles = self.zRoles.getManagedRoles().getPermanentRoleNames()
        self.failUnless( renamedRoles == roles )

    def tearDown(self):
        self.user.unbind()

Globals.InitializeClass(testZRoles)

if __name__ == '__main__':
    
    tests = unittest.makeSuite(testZRoles, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
