"""

    Module: $RCSfile: TestZSchema.py,v $
    Author: $Author: payne $
    Date:   $Date: 2003/12/15 00:47:10 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.45 $'[11:-2]

import os, sys
import ZopeEnviron

import Zope
import ZODB
import ZServer

from Acquisition import Implicit
from OFS.Folder import manage_addFolder, Folder
from ZPublisher.Request import Request
from ZPublisher.Response import Response

from copy import deepcopy
from string import index
import unittest

from Connection import AuthenticatedUser
from LDAPExceptions import REQUIRED_ATTRIBUTE, NO_SUCH_ATTRIBUTE
from LDAPUtilities import listsAreEquivalent
from ManagedRoles import ManagedRoles
from ManagedSchema import ManagedSchema
from RequestTest import request
from SchemaDefaults_test import *
from TestDirectory import getTestDirectory
from ZLDAPTree import ZLDAPTree
from ZSchema import ZSchema

app = Zope.app()

class testZSchema(unittest.TestCase,Folder):

    __ac_roles__ = ('Internal')

    def setUp(self):
        directory = getTestDirectory()
	self.response = Response(stdout=sys.stdout,stderr=sys.stderr)
	REQUEST = Request(sys.stdin, os.environ, self.response )
        REQUEST.set('objectCategory', 'person')
        self.user = AuthenticatedUser('testadmin', 'testadmin',
                       directory)
        self.ZSchema = ZSchema( self.user, REQUEST )
        self.schema = self.ZSchema._schema
	self = self.__of__(app)
	manage_addFolder( self, 'testFolder' )

    def test01(self):
        """ 01 TestZSchema: testing checkObjectClassCompleteness """
        self.failUnless( self.ZSchema.checkObjectClassCompleteness(
                         ['top', 'person', 'organizationalPerson'] ) )
        self.failIf( self.ZSchema.checkObjectClassCompleteness(
                     ['top', 'person', 'inetOrgPerson'] ) )

    def test02(self):
        """ 02 TestZSchema: getManagedRoles """
        managedRoles = self.ZSchema.getManagedRoles()
        roles = self.user.getLDAPDirectory().getManagedRoles()
        self.failUnless( managedRoles == roles )

    def test03(self):
        """ 03 TestZSchema: getManagedSchema """
        schema = self.ZSchema.getManagedSchema()
        self.failUnless( schema == self.schema )

    def test04(self):
        """ 04 TestZSchema: update from directory """
        self.ZSchema.updateSchemaFromLDAP( self.user )

        allowed = [
            'audio', 'businessCategory', 'carLicense',
            'cn', 'departmentNumber', 'description',
            'destinationIndicator', 'displayName', 'employeeNumber',
            'employeeType', 'facsimileTelephoneNumber',
            'givenName', 'homePhone', 'homePostalAddress',
            'initials', 'internationaliSDNNumber', 'jpegPhoto',
            'l', 'labeledUri', 'mail', 'manager', 'mobile',
            'o', 'objectClass', 'ou', 'pager', 'photo', 
	    'physicalDeliveryOfficeName', 'postOfficeBox', 
	    'postalAddress', 'postalCode', 'preferredDeliveryMethod', 
	    'preferredLanguage', 'registeredAddress', 'roomNumber', 
	    'secretary', 'seeAlso', 'sn', 'st', 'street', 
	    'telephoneNumber', 'teletexTerminalIdentifier', 
	    'telexNumber', 'title', 'uid', 'userCertificate', 
	    'userPKCS12', 'userPassword', 'userSMIMECertificate', 
	    'x121Address', 'x500UniqueIdentifier', ]

        allowedOpenLDAP = [
            'audio', 'businessCategory', 'carLicense',
            'cn', 'departmentNumber', 'description',
            'destinationIndicator', 'displayName', 'employeeNumber',
            'employeeType', 'facsimileTelephoneNumber',
            'givenName', 'homePhone', 'homePostalAddress',
            'initials', 'internationaliSDNNumber', 'jpegPhoto',
            'l', 'labeledURI', 'mail', 'manager', 'mobile',
            'o', 'objectClass', 'ou', 'pager', 'photo',
            'physicalDeliveryOfficeName', 'postOfficeBox',
            'postalAddress', 'postalCode', 'preferredDeliveryMethod',
            'preferredLanguage', 'registeredAddress', 'roomNumber',
            'secretary', 'seeAlso', 'sn', 'st', 'street',
            'telephoneNumber', 'teletexTerminalIdentifier',
            'telexNumber', 'title', 'uid', 'userCertificate',
            'userPKCS12', 'userPassword', 'userSMIMECertificate',
            'x121Address', 'x500uniqueIdentifier', ]

        schema = self.user.getLDAPDirectory().getPersonSchema()
        found = schema.getAllowedAttributeNames()
        if 'labeledUri' in found:
            for item in allowed:
                self.failUnless( item in found )
                found.remove( item )
        else:
            for item in allowedOpenLDAP:
                self.failUnless( item in found )
                found.remove( item )
        self.failUnless( found == [])

        found = schema.getAttributeCollection()
        self.failUnless( listsAreEquivalent( 
                         schema.getAllowedAttributeNames(), found.keys()))

        attrSchema = schema.getAttributeSchema( 'cn' )
        self.failUnless( attrSchema.getRequired() )

        self.assertRaises(AssertionError,
                          schema.getAttributeSchema,
                          'badValue')

        available = [
            'audio', 'destinationIndicator', 'employeeNumber', 
	    'internationaliSDNNumber', 'jpegPhoto', 'labeledUri', 
	    'photo', 'preferredDeliveryMethod', 'preferredLanguage', 
	    'registeredAddress', 'teletexTerminalIdentifier', 
	    'telexNumber', 'userCertificate', 'userPKCS12', 
	    'userSMIMECertificate', 'x121Address', 
	    'x500UniqueIdentifier', ]

        availableOpenLDAP = [
            'audio', 'destinationIndicator', 'employeeNumber',
            'internationaliSDNNumber', 'jpegPhoto', 'labeledURI',
            'photo', 'preferredDeliveryMethod', 'preferredLanguage',
            'registeredAddress', 'teletexTerminalIdentifier',
            'telexNumber', 'userCertificate', 'userPKCS12',
            'userSMIMECertificate', 'x121Address',
            'x500uniqueIdentifier', ]

        found = schema.getAvailableAttributeNames()
        if 'labeledUri' in found:
            for item in available:
                self.failUnless( item in found )
                found.remove( item )
        else:
            for item in availableOpenLDAP:
                self.failUnless( item in found )
                found.remove( item )
        self.failUnless( found == [] )

    def test05(self):
        """ 05 TestZSchema: update from LDAP makes naming attribute required """
        schema = self.ZSchema.getManagedSchema()
        schema.setNamingAttribute('postalAddress')
        self.ZSchema.updateSchemaFromLDAP(self.user)
        self.failUnless( schema.getAttributeSchema( 'postalAddress' ).getRequired() )

    def test06(self):
        """ 06 TestZSchema: testing Object Classes form """
        self.ZSchema.updateSchemaFromLDAP( self.user )
        unmanaged = [
            'account', 'alias', 'applicationentity', 'applicationprocess', 
            'automount', 'bootabledevice', 'cacheobject', 'calentry', 
            'certificationauthority', 'certificationauthority-v2', 
            'changelogentry', 'cirreplicasource', 'cosclassicdefinition', 
            'cosdefinition', 'cosindirectdefinition', 'cospointerdefinition', 
            'cossuperdefinition', 'costemplate', 'country', 
            'crldistributionpoint', 'dcobject', 'device', 
            'directoryserverfeature', 'dmd', 'dnsdomain', 'document', 
            'documentseries', 'domain', 'domainrelatedobject', 
            'ds-signedplugin', 'dsa', 'dsattributeencryption', 
            'dsfilterspconfigelement', 'dsfilterspconfigset', 
            'dsfilterspfractionelement', 'dsfilterspsparseelement', 
            'dsfilterspsubtreeelement', 'dsidentitymapping', 
            'dsidmappinglibrary', 'dspatternmatching', 'dssaslconfig', 
            'extensibleobject', 'friendlycountry', 'glue', 
            'groupofcertificates', 'groupofmailenhanceduniquenames', 
            'groupofnames', 'groupofuniquenames', 'groupofurls', 
            'ieee802device', 'inetadmin', 'inetdomain', 'inetorgperson', 
            'inetsubscriber', 'inetuser', 'iphost', 'iplanetlinkedorganization',
            'iplanetorgservice', 'iplanetpreferences', 
            'iplanetreversiblepasswordobject', 'iplanetservice', 
            'iplanetservicecomponent', 'iplanetserviceplugin', 
            'ipnetwork', 'ipprotocol', 'ipservice', 'javacontainer', 
            'javamarshalledobject', 'javanamingreference', 'javaobject', 
            'javaserializedobject', 'kerberossecurityobject', 
            'labeleduriobject', 'ldapreplica', 'ldapserver', 'ldapsubentry', 
            'locality', 'mailgroup', 'mailgroupmanagement', 
            'mailgroupmanagement-globalconfig', 'mailgroupmember', 
            'mailrecipient', 'netscapecalendarserver', 
            'netscapecertificateserver', 'netscapecompassserver', 
            'netscapedirectoryserver', 'netscapemachinedata', 
            'netscapemailserver', 'netscapemediaserver', 
            'netscapemessagingserver', 'netscapenewsserver', 
            'netscapepreferencemap', 'netscapeproxyserver', 
            'netscapeserver', 'netscapewebserver', 'netshareaccount', 
            'netshareprojectmanagementglobalconfig', 'newpilotperson', 
            'nginfo', 'nismap', 'nisnetgroup', 'nisobject', 'nsadminconfig', 
            'nsadminconsoleuser', 'nsadmindomain', 'nsadminglobalparameters', 
            'nsadmingroup', 'nsadminobject', 'nsadminresourceeditorextension', 
            'nsadminserver', 'nsapplication', 'nsbackendinstance', 
            'nsbrowserclient', 'nsbrowsersecurity', 'nscaladmin', 
            'nscalresource', 'nscaluser', 'nscertificateserver', 
            'nschangelog4config', 'nscomplexroledefinition', 
            'nsconfig', 'nsconsumer4config', 'nscontainer', 'nscustomview', 
            'nsdefaultobjectclasses', 'nsdirectoryinfo', 'nsdirectoryserver', 
            'nsds5replica', 'nsds5replicationagreement', 'nsencryptionconfig', 
            'nsencryptionmodule', 'nsfilteredroledefinition', 
            'nsglobalparameters', 'nshost', 'nsindex', 'nslegacyadmingroup', 
            'nslegacyadminserver', 'nslegacyapplication', 'nslegacyserver', 
            'nslicenseuser', 'nsliprofile', 'nsliprofileelement', 'nsliptr', 
            'nsliserver', 'nsmailclient', 'nsmailsecurity', 'nsmanageddept', 
            'nsmanageddeptadmingroup', 'nsmanageddomain', 
            'nsmanagedfamilygroup', 'nsmanagedisp', 'nsmanagedmaillist', 
            'nsmanagedorgunit', 'nsmanagedpeoplecontainer', 'nsmanagedperson', 
            'nsmanagedroledefinition', 'nsmappingtree', 'nsmcdconfig', 
            'nsmessagingserveruser', 'nsmsgcfgalarm', 'nsmsgcfgalarmcontainer', 
            'nsmsgcfgalias', 'nsmsgcfgcontainer', 'nsmsgcfgexpirerule', 
            'nsmsgcfggen', 'nsmsgcfghttp', 'nsmsgcfgimap', 'nsmsgcfglog', 
            'nsmsgcfgmta', 'nsmsgcfgmtaaccept', 'nsmsgcfgmtaautoreplyhandler', 
            'nsmsgcfgmtaerrorhandler', 'nsmsgcfgmtalog', 
            'nsmsgcfgmtamboxdeliver', 'nsmsgcfgmtaprogdeliver', 
            'nsmsgcfgmtarouter', 'nsmsgcfgmtasmtpdeliver', 
            'nsmsgcfgmtaunixdeliver', 'nsmsgcfgnntp', 'nsmsgcfgpartition', 
            'nsmsgcfgpop', 'nsmsgcfgpublic', 'nsmsgcfgreport', 
            'nsmsgcfgscript', 'nsmsgcfgservice', 'nsmsgcfgsnmp', 
            'nsmsgcfgstore', 'nsmsgcfguser', 'nsnestedroledefinition', 
            'nsresourceref', 'nsroledefinition', 'nssimpleroledefinition', 
            'nsslapdconfig', 'nsslapdplugin', 'nssnmp', 'nstask', 
            'nstaskgroup', 'nstombstone', 'nstopologycustomview', 
            'nstopologyplugin', 'nsuniquenessdomain', 'nsvalueitem', 
            'nswcaluser', 'ntgroup', 'ntuser', 'oncrpc', 'organization', 
            'organizationalperson', 'organizationalrole', 'organizationalunit', 
            'passwordobject', 'passwordpolicy', 'personalinterestprofile', 
            'pilotobject', 'pilotorganization', 'pipuser', 'pipuserinfo', 
            'posixaccount', 'posixgroup', 'referral', 'residentialperson', 
            'rfc822localpart', 'room', 'shadowaccount', 'simplesecurityobject', 
            'strongauthenticationuser', 'stsciorganizationalunit', 
            'stsciperson', 'subschema', 'usersecurityinformation', 
            'vlvindex', 'vlvsearch', ]

        unmanagedOpenLDAP = [
            'account', 'alias', 'applicationentity', 'applicationprocess',
            'certificationauthority', 'certificationauthority-v2',
            'country', 'crldistributionpoint', 'dcobject', 'deltacrl',
            'device', 'dmd', 'dnsdomain', 'document', 'documentseries',
            'domain', 'domainrelatedobject', 'dsa', 'extensibleobject',
            'friendlycountry', 'groupofnames', 'groupofuniquenames',
            'inetorgperson', 'labeleduriobject', 'locality', 'monitor',
            'openldaprootdse', 'organization', 'organizationalperson',
            'organizationalrole', 'organizationalunit', 'pilotdsa',
            'pilotorganization', 'pilotperson', 'pkica', 'pkiuser',
            'qualitylabelleddata', 'referral', 'residentialperson',
            'rfc822localpart', 'room', 'simplesecurityobject',
            'strongauthenticationuser', 'subentry', 'subschema',
            'uidobject', 'usersecurityinformation', ]

        found = self.ZSchema.getUnmanagedObjectClasses()
        if 'iplanetservice' in found:
            self.failUnless (listsAreEquivalent(unmanaged, found))
        else:
            self.failUnless (listsAreEquivalent(unmanagedOpenLDAP, found))

	selectedObjectClasses = [
	    'top', 'person', 'organizationalPerson', 'inetOrgPerson' ]
	objectClassAttr = self.ZSchema._schema.getAttributeSchema('objectClass')
	required = objectClassAttr.getRequiredValues()
	valid = deepcopy(objectClassAttr.getValidValues())
	self.failUnless(listsAreEquivalent(selectedObjectClasses,required))

	for item in required:
	    valid.remove(item)
	self.failUnless( valid == [] )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("managed",1)
	REQUEST.set("required",1)
	REQUEST.set("syntax","1.3.6.1.4.1.1466.115.121.1.38")
	REQUEST.set("updateSchema",1)
	REQUEST.set("objectCategory","person")
	REQUEST.set("schemaFormID","schemaObjectClasses")
	REQUEST.set("ldapName","objectClass")
	REQUEST.set("friendlyName","Object Class")
	REQUEST.set("displayCategories",['General', 'Telephone', 
		'Postal Mail', 'E-Mail', 'Organization', 'System', 'All'])
	REQUEST.set("readers",['ANON', 'AUTH'])
	REQUEST.set("writers",['System Administrators'])
	REQUEST.set("pointsTo","")

	REQUEST.set("newClasses",['locality'])
	REQUEST.set("addAvailableObjectClasses"," Add ")
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless(index(message,"Added object classes ['locality'].")==0)
	objectClassAttr = zschema._schema.getAttributeSchema('objectClass')
	valid = objectClassAttr.getValidValues()
        self.failIf( 'locality' in self.ZSchema.getUnmanagedObjectClasses())
	self.failUnless( 'locality' in valid )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("managed",1)
	REQUEST.set("required",1)
	REQUEST.set("syntax","1.3.6.1.4.1.1466.115.121.1.38")
	REQUEST.set("updateSchema",1)
	REQUEST.set("objectCategory","person")
	REQUEST.set("schemaFormID","schemaObjectClasses")
	REQUEST.set("ldapName","objectClass")
	REQUEST.set("friendlyName","Object Class")
	REQUEST.set("displayCategories",['General', 'Telephone', 
		'Postal Mail', 'E-Mail', 'Organization', 'System', 'All'])
	REQUEST.set("readers",['ANON', 'AUTH'])
	REQUEST.set("writers",['System Administrators'])
	REQUEST.set("pointsTo","")

	REQUEST.set("selectClasses",['locality'])
	REQUEST.set("moveObjectClassesToSelected",">>")
        zschema = ZSchema( self.user, REQUEST )
	zschema = zschema.__of__(self.testFolder)
        message = zschema.updateSchemaFromRequest()
	self.failUnless(index(message,"Updated object class lists.  Updated schema attributes.")==0)
	objectClassAttr = zschema._schema.getAttributeSchema('objectClass')
	required = objectClassAttr.getRequiredValues()
	self.failUnless( 'locality' in required )
        self.failIf( 'locality' in self.ZSchema.getUnmanagedObjectClasses())

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("managed",1)
	REQUEST.set("required",1)
	REQUEST.set("syntax","1.3.6.1.4.1.1466.115.121.1.38")
	REQUEST.set("updateSchema",1)
	REQUEST.set("objectCategory","person")
	REQUEST.set("schemaFormID","schemaObjectClasses")
	REQUEST.set("ldapName","objectClass")
	REQUEST.set("friendlyName","Object Class")
	REQUEST.set("displayCategories",['General', 'Telephone', 
		'Postal Mail', 'E-Mail', 'Organization', 'System', 'All'])
	REQUEST.set("readers",['ANON', 'AUTH'])
	REQUEST.set("writers",['System Administrators'])
	REQUEST.set("pointsTo","")

	REQUEST.set("availClasses",['locality'])
	REQUEST.set("moveObjectClassesToAvailable","<<")
        zschema = ZSchema( self.user, REQUEST )
	zschema = zschema.__of__(self.testFolder)
        message = zschema.updateSchemaFromRequest()
	self.failUnless(index(message,"Dropped specified objectClasses.  Updated schema attributes.")==0)
	objectClassAttr = zschema._schema.getAttributeSchema('objectClass')
	required = objectClassAttr.getRequiredValues()
	self.failIf( 'locality' in required )
	valid = objectClassAttr.getValidValues()
	self.failUnless( 'locality' in valid )
        self.failIf( 'locality' in self.ZSchema.getUnmanagedObjectClasses())

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("managed",1)
	REQUEST.set("required",1)
	REQUEST.set("syntax","1.3.6.1.4.1.1466.115.121.1.38")
	REQUEST.set("updateSchema",1)
	REQUEST.set("objectCategory","person")
	REQUEST.set("schemaFormID","schemaObjectClasses")
	REQUEST.set("ldapName","objectClass")
	REQUEST.set("friendlyName","Object Class")
	REQUEST.set("displayCategories",['General', 'Telephone', 
		'Postal Mail', 'E-Mail', 'Organization', 'System', 'All'])
	REQUEST.set("readers",['ANON', 'AUTH'])
	REQUEST.set("writers",['System Administrators'])
	REQUEST.set("pointsTo","")

	REQUEST.set("selectClasses",['locality'])
	REQUEST.set("dropObjectClasses","Drop")
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless(index(message,"Dropped specified objectClasses.")==0)
	objectClassAttr = zschema._schema.getAttributeSchema('objectClass')
	valid = objectClassAttr.getValidValues()
	self.failIf( 'locality' in valid )
        self.failUnless( 'locality' in self.ZSchema.getUnmanagedObjectClasses())

    def test07(self):
        """ 07 TestZSchema: testing Select Attributes form """
        
        schema = self.ZSchema.getManagedSchema()
	required = ['cn', 'objectClass', 'sn', 'uid' ]
	found = schema.getRequiredAttributeNames()
	self.failUnless( listsAreEquivalent( required, found ))

        managed = schema.getManagedAttributeNames()
	available = schema.getAvailableAttributeNames()
	self.failUnless( 'audio' in available )
	self.failIf( 'audio' in managed )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("updateSchema",1)
	REQUEST.set("objectCategory","person")
	REQUEST.set("schemaFormID","schemaManaged")
	REQUEST.set("manageAttrs",['audio'])
	REQUEST.set("moveAttributeToManaged",">>")
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless(index(message,"Updated attribute lists.")==0)
        schema = zschema.getManagedSchema()
        managed = schema.getManagedAttributeNames()
	available = schema.getAvailableAttributeNames()
	self.failUnless( 'audio' in managed )
	self.failIf( 'audio' in available )
	self.failIf( 'cn' in schema.getManagedNotRequiredAttributeNames() )
	self.failIf( 'cn' in available )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("updateSchema",1)
	REQUEST.set("objectCategory","person")
	REQUEST.set("schemaFormID","schemaManaged")
	REQUEST.set("unmanageAttrs",['audio'])
	REQUEST.set("moveAttributeToAvailable",">>")
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless(index(message,"Updated attribute lists.")==0)
        schema = zschema.getManagedSchema()
        managed = schema.getManagedAttributeNames()
	available = schema.getAvailableAttributeNames()
	self.failUnless( 'audio' in available )
	self.failIf( 'audio' in managed )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("updateSchema",1)
	REQUEST.set("objectCategory","person")
	REQUEST.set("schemaFormID","schemaManaged")
	REQUEST.set("manageAttrs",['bogusAttr'])
	REQUEST.set("moveAttributeToManaged",">>")
        zschema = ZSchema( self.user, REQUEST )
        self.assertRaises(NO_SUCH_ATTRIBUTE,
        		  zschema.updateSchemaFromRequest )

    def test08(self):
        """ 08 TestZSchema: testing Naming Attributes form """
        schema = self.ZSchema.getManagedSchema()
	namingAttribute = schema.getNamingAttribute()
	self.failUnless( namingAttribute == 'uid' )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("updateSchema",1)
	REQUEST.set("objectCategory","person")
	REQUEST.set("schemaFormID","schemaNamingAttribute")
	REQUEST.set("namingAttribute",'cn')
	REQUEST.set("setNamingAttribute"," Set ")
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( index(message,'Set naming attribute.') == 0 )
        schema = zschema.getManagedSchema()
	namingAttribute = schema.getNamingAttribute()
	self.failUnless( namingAttribute == 'cn' )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("updateSchema",1)
	REQUEST.set("objectCategory","person")
	REQUEST.set("schemaFormID","schemaNamingAttribute")
	REQUEST.set("namingAttribute",'uid')
	REQUEST.set("setNamingAttribute"," Set ")
        zschema = ZSchema( self.user, REQUEST )
        schema = zschema.getManagedSchema()
        message = zschema.updateSchemaFromRequest()

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set("updateSchema",1)
	REQUEST.set("objectCategory","person")
	REQUEST.set("schemaFormID","schemaNamingAttribute")
	REQUEST.set("namingAttribute",'bogusAttribute')
	REQUEST.set("setNamingAttribute"," Set ")
        zschema = ZSchema( self.user, REQUEST )
        schema = zschema.getManagedSchema()
        self.assertRaises(NO_SUCH_ATTRIBUTE,
        		  zschema.updateSchemaFromRequest )

    def test09(self):
        """ 09 TestZSchema: testing Display Categories form """
        schema = self.ZSchema.getManagedSchema()
	categories = schema.getDisplayCategories()
	self.failUnless( 'General' in categories )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaCategories"
	REQUEST.form["ids"] = ['General']
	REQUEST.form["changeDisplayCategories"] = " Delete "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
        schema = zschema.getManagedSchema()
	self.failUnless(index(message,"Deleted object categories.")==0)
	categories = schema.getDisplayCategories()
	self.failIf( 'General' in categories )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaCategories"
	REQUEST.form["newCategory"] = 'General'
	REQUEST.form["changeDisplayCategories"] = " Add "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
        schema = zschema.getManagedSchema()
	self.failUnless(index(message,"Added object category.")==0)
	categories = schema.getDisplayCategories()
	self.failUnless( categories[-1] == 'General' )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaCategories"
	REQUEST.form["ids"] = ['General']
	REQUEST.form["changeDisplayCategories"] = " First "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
        schema = zschema.getManagedSchema()
	self.failUnless(index(message,"Re-ordered display categories.")==0)
	categories = schema.getDisplayCategories()
	self.failUnless( categories[0] == 'General' )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaCategories"
	REQUEST.form["ids"] = ['General']
	REQUEST.form["renameDisplayCategories"] = " Rename "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
        schema = zschema.getManagedSchema()
	request = zschema.request
	self.failUnless(request.get('schemaFormID')=='schemaRenameCategories')

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["schemaFormID"] = "schemaCategories"
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["renameDisplayCategories"] = " Rename "
	REQUEST.form["haveNewNames"] = 1
	REQUEST.form["ids"] = ['General']
	REQUEST.form["new_ids"] = ['NewGeneral']
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
        schema = zschema.getManagedSchema()
	self.failUnless(index(message,"Renamed display categories")==0)
	categories = schema.getDisplayCategories()
	self.failUnless( categories[0] == 'NewGeneral' )
	cnSchema = schema.getAttributeSchema('cn')
	self.failUnless('NewGeneral' in cnSchema.getDisplayCategories())

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["schemaFormID"] = "schemaCategories"
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["renameDisplayCategories"] = " Rename "
	REQUEST.form["haveNewNames"] = 1
	REQUEST.form["ids"] = ['NewGeneral']
	REQUEST.form["new_ids"] = ['General']
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
        schema = zschema.getManagedSchema()
	categories = schema.getDisplayCategories()
	self.failUnless( categories[0] == 'General' )
	cnSchema = schema.getAttributeSchema('cn')
	self.failUnless('General' in cnSchema.getDisplayCategories())

	objectClassAttr = schema.getAttributeSchema('objectClass')
	self.failIf('General' in objectClassAttr.getDisplayCategories())
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaAttrsByCategory"
	REQUEST.form["displayCategory"] = "General"
	REQUEST.form["ids"] = ['cn','departmentNumber','description',
		'displayName','mail','roomNumber','sn',
		'telenphoneNumber','title','userPassword',
		'objectClass']
	REQUEST.form["saveAttrsByCategory"] = " Save "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
        schema = zschema.getManagedSchema()
	self.failUnless(index(message,"Saved display category \"General\"'s presence")==0)
	objectClassAttr = schema.getAttributeSchema('objectClass')
	self.failUnless('General' in objectClassAttr.getDisplayCategories())
	cnSchema = schema.getAttributeSchema('cn')
	self.failUnless('General' in cnSchema.getDisplayCategories())

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaAttrsByCategory"
	REQUEST.form["displayCategory"] = "General"
	REQUEST.form["ids"] = ['cn','departmentNumber','description',
		'displayName','mail','roomNumber','sn',
		'telenphoneNumber','title','userPassword',]
	REQUEST.form["saveAttrsByCategory"] = " Save "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
        schema = zschema.getManagedSchema()
	self.failUnless(index(message,"Saved display category \"General\"'s presence")==0)
	objectClassSchema = schema.getAttributeSchema('objectClass')
	self.failIf('General' in objectClassSchema.getDisplayCategories())
	cnSchema = schema.getAttributeSchema('cn')
	self.failUnless('General' in cnSchema.getDisplayCategories())

    def test10(self):
        """ 10 TestZSchema: testing Manage Attributes form """
        schema = self.ZSchema.getManagedSchema()
	cnSchema  = schema.getAttributeSchema('cn')
	self.failUnless( cnSchema.getRequired() )
	self.failUnless( cnSchema.getFriendlyName() == 'Common Name(s)' )
	self.failUnless( 'General' in cnSchema.getDisplayCategories() )
	self.failUnless( 'ANON' in cnSchema.getReaders() )
	self.failUnless( 'SELF' in cnSchema.getWriters() )
	self.failUnless( cnSchema.getValidValues() == [] )
	self.failUnless( cnSchema.getRequiredValues() == [] )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["managed"] = 1
	REQUEST.form["required"] = 1
	REQUEST.form["syntax"] = "1.3.6.1.4.1.1466.115.121.1.15{32768}"
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaAttributes"
	REQUEST.form["ldapName"] = "cn"
	REQUEST.form["singleValued"] = "1"
	REQUEST.form["friendlyName"] = "New Common Name(s)"
	REQUEST.form["description"] = "The person's common name."
	REQUEST.form["displayCategories"] = ['Personal', 'All' ]
	REQUEST.form["readers"] = ['ANON', 'AUTH']
	REQUEST.form["writers"] = ['SELF', 'Divisional Administrators',
		'System Administrators']
	REQUEST.form["validValues"] = ['My Name']
	REQUEST.form["requiredValues"] = ['My Name','Some other name']
	REQUEST.form["saveAttributeSchema"] = " Save "
        zschema = ZSchema( self.user, REQUEST )
	zschema = zschema.__of__(self.testFolder)
        message = zschema.updateSchemaFromRequest()
	self.failUnless(index(message,"Updated schema attributes.")==0)
        schema = zschema.getManagedSchema()
	cnSchema  = schema.getAttributeSchema('cn')
	self.failUnless( cnSchema.getRequired() )
	self.failIf( cnSchema.getSingleValued() )
	self.failUnless( cnSchema.getFriendlyName() == 'New Common Name(s)' )
	self.failIf( 'General' in cnSchema.getDisplayCategories() )
	self.failIf( 'SELF' in cnSchema.getReaders() )
	self.failUnless( 'SELF' in cnSchema.getWriters() )
	self.failUnless( cnSchema.getValidValues() == ['My Name'] )
	self.failUnless( cnSchema.getRequiredValues() == ['My Name'] )
	self.failUnless(index(zschema.nextAttributeLink(),"ldapName=departmentNumber")>0)
	self.failUnless(index(zschema.prevAttributeLink(),"ldapName=carLicense")>0)

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["managed:int"] = 1
	REQUEST.form["required:int"] = 1
	REQUEST.form["syntax"] = "1.3.6.1.4.1.1466.115.121.1.15{32768}"
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaAttributes"
	REQUEST.form["ldapName"] = "cn"
	REQUEST.form["singleValued"] = "0"
	REQUEST.form["friendlyName"] = "Common Name(s)"
	REQUEST.form["description"] = "The person's common name."
	REQUEST.form["displayCategories"] = ['General', 'Personal', 'All' ]
	REQUEST.form["readers"] = ['ANON', 'AUTH']
	REQUEST.form["writers"] = ['SELF', 'Divisional Administrators',
		'System Administrators']
	REQUEST.form["validValues"] = []
	REQUEST.form["saveAttributeSchema"] = " Save "
        zschema = ZSchema( self.user, REQUEST )
	zschema = zschema.__of__(self.testFolder)
        message = zschema.updateSchemaFromRequest()
        schema = zschema.getManagedSchema()

    def test11(self):
        """ 11 TestZSchema: testing Access by Role form """

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaAttrsByRole"
	REQUEST.form["roleName"] = "ANON"
        zschema = ZSchema( self.user, REQUEST )
	self.failUnless(index(zschema.nextRoleLink(),"roleName=AUTH")>0)
	self.failUnless(index(zschema.prevRoleLink(),"roleName=SELF")>0)

    def test12(self):
        """ 12 TestZSchema: testing Add/Delete by Role form """
        
	REQUEST = Request(sys.stdin, os.environ, self.response )
        REQUEST.set( 'updateSchema', '1',)
        REQUEST.set( 'objectCategory', 'person',)
        REQUEST.set( 'schemaFormID', 'schemaControllers',)
        REQUEST.set( 'newRoles', ['Internal',],)
        REQUEST.set( 'addControllers', ' Add ',)
        zschema = ZSchema( self.user, REQUEST )
        zschema.updateSchemaFromRequest()
        schema = self.user.getLDAPDirectory().getPersonSchema()
	self.failUnless( 'Internal' in schema.getControllers() )
	namingAttribute = schema.getNamingAttribute()
	namingAttrSchema = schema.getAttributeSchema(namingAttribute)
	self.failUnless( 'Internal' in namingAttrSchema.getWriters() )

	REQUEST = Request(sys.stdin, os.environ, self.response )
        REQUEST.set( 'updateSchema', '1',)
        REQUEST.set( 'objectCategory', 'person',)
        REQUEST.set( 'schemaFormID', 'schemaControllers',)
        REQUEST.set( 'ids', ['Internal',],)
        REQUEST.set( 'deleteControllers', ' Delete ',)
        zschema = ZSchema( self.user, REQUEST )
        zschema.updateSchemaFromRequest()
        schema = self.user.getLDAPDirectory().getPersonSchema()
	self.failIf( 'Internal' in schema.getControllers() )
	self.failIf( 'Internal' in namingAttrSchema.getWriters() )

    def test12(self):
        """ 12 TestZSchema: testing Search Filter form """
        
	searchFilterOrig = self.schema.getSearchFilter()
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaSearchFilter"
	REQUEST.form["searchFilter"] = "test search filter"
	REQUEST.form["searchFilterSave"] = " Save Changes "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless(index(message,"Set search filter.")==0)
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getSearchFilter()=="test search filter" )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaSearchFilter"
	REQUEST.form["searchFilter"] = searchFilterOrig
	REQUEST.form["searchFilterSave"] = " Save Changes "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getSearchFilter()==searchFilterOrig )

    def test13(self):
        """ 13 TestZSchema: testing Search Keys form """

	# Add some search keys, change name for the one we have.
	# Try setting sort key to a bogus value.
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaSearchKeys"
	REQUEST.form["searchSortKey"] = "bogusKey"
	REQUEST.form["uid_name"] = "Account Name"
	REQUEST.form["searchKeys"] = ["displayName","telephoneNumber",
		"roomNumber","departmentNumber","mail",
		"physicalDeliveryOfficeName"]
	REQUEST.form["addSearchKeys"] = " Add "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( index(message,"Added search key(s).") == 0 )
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getSearchSortKey() == 'uid' )
	self.failUnless( schema.getSearchKeyName('uid') == "Account Name" )

	# Supply names for new keys, make displayName the sort key and 
	# move displayName and telephoneNumber to the top of the list.
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaSearchKeys"
	REQUEST.form["searchSortKey"] = "displayName"
	REQUEST.form["displayName_name"] = "Display Name"
	REQUEST.form["telephoneNumber_name"] = "Telephone Number"
	REQUEST.form["roomNumber_name"] = "Office Number"
	REQUEST.form["departmentNumber_name"] = "Branch"
	REQUEST.form["mail_name"] = "Email Address"
	REQUEST.form["physicalDeliveryOfficeName_name"] = "Mail Stop"
	REQUEST.form["uid_name"] = "Account name"
	REQUEST.form["ids"] = ["displayName","telephoneNumber"]
	REQUEST.form["firstSearchKeys"] = " First "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( index(message,"Reordered search keys") == 0 )
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getSearchKeys()[0] == 'displayName' )
	self.failUnless( schema.getSearchKeys()[1] == 'telephoneNumber' )
	self.failUnless( schema.getSearchSortKey() == 'displayName' )
	self.failUnless( schema.getSearchKeyName('mail') == "Email Address" )

	# Delete the new keys, which includes the sort key.  Make
	# sure the sort key is reassigned to the remaining key.
	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaSearchKeys"
	REQUEST.form["searchSortKey"] = "displayName"
	REQUEST.form["ids"] = ['displayName','telephoneNumber',
		'roomNumber','departmentNumber','mail',
		'physicalDeliverOfficeName']
	REQUEST.form["displayName_name"] = "Display Name"
	REQUEST.form["telephoneNumber_name"] = "Telephone Number"
	REQUEST.form["roomNumber_name"] = "Office Number"
	REQUEST.form["departmentNumber_name"] = "Branch"
	REQUEST.form["mail_name"] = "Email Address"
	REQUEST.form["physicalDeliveryOfficeName_name"] = "Mail Stop"
	REQUEST.form["uid_name"] = "Username"
	REQUEST.form["deleteSearchKeys"] = " Delete "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( index(message,"Deleted search keys") == 0 )
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getSearchSortKey() == 'uid' )
	self.failUnless( schema.getSearchKeyName('uid') == "Username" )

    def test14(self):
        """ 14 TestZSchema: testing Browsing form """

	# We try to make the browsing label be "givenName (displayName)"
	# Add the givenName key and change the format string.

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaBrowsing"
	REQUEST.form["browseFormat"] = "%s %s"
	REQUEST.form["browseSortKey"] = "displayName"
	REQUEST.form["browseKeys"] = ['givenName']
	REQUEST.form["addBrowseKeys"] = " Add "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( index(message,"Added attributes") == 0 )
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getBrowseSortKey() == 'displayName' )
	self.failUnless( schema.getBrowseFormat() == '%s %s' )
	self.failUnless( 'givenName' in schema.getBrowseKeys() )
	self.failUnless( 'displayName' in schema.getBrowseKeys() )

	# Rearrange browse keys, change sort key and format

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaBrowsing"
	REQUEST.form["browseFormat"] = "%s (%s)"
	REQUEST.form["browseSortKey"] = "givenName"
	REQUEST.form["ids"] = ['givenName']
	REQUEST.form["firstBrowseKeys"] = " First "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( index(message,"Reordered browse keys.") == 0 )
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getBrowseSortKey() == 'givenName' )
	self.failUnless( schema.getBrowseFormat() == '%s (%s)' )
	self.failUnless( schema.getBrowseKeys()[0] == 'givenName' )
	self.failUnless( schema.getBrowseKeys()[1] == 'displayName' )

	# Does it work?  Instantiate a ZLDAPTree object and get a
	# listing of some search results.

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.set('user',self.user)
	REQUEST.set('tree-level',1)
	REQUEST.set('tree-colspan',3)
	REQUEST.set('mappingDN','ou=ST ScI,o=test')
	zLDAPTree = ZLDAPTree()
	results = zLDAPTree.orgUnitMemberBrowser(REQUEST,REQUEST,
		standard_html_header='', standard_html_footer='')
	self.failUnless(index(results,"Anne (User, Anne)")>0)

	# First with empty ids should not rearrange

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaBrowsing"
	REQUEST.form["browseFormat"] = "%s (%s)"
	REQUEST.form["browseSortKey"] = "givenName"
	REQUEST.form["ids"] = []
	REQUEST.form["deleteBrowseKeys"] = " Delete "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( message == "" )
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getBrowseSortKey() == 'givenName' )
	self.failUnless( schema.getBrowseFormat() == '%s (%s)' )
	self.failUnless( schema.getBrowseKeys()[0] == 'givenName' )
	self.failUnless( schema.getBrowseKeys()[1] == 'displayName' )

	# Delete givenName, set format back.

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaBrowsing"
	REQUEST.form["browseFormat"] = "%s"
	REQUEST.form["browseSortKey"] = "givenName"
	REQUEST.form["ids"] = ['givenName']
	REQUEST.form["deleteBrowseKeys"] = " Delete "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( index(message,"Deleted attributes ['givenName'].") == 0 )
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getBrowseSortKey() == 'displayName' )
	self.failUnless( schema.getBrowseFormat() == '%s' )
	self.failUnless( schema.getBrowseKeys() == ['displayName'] )

	# Delete with empty ids should not delete

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaBrowsing"
	REQUEST.form["browseFormat"] = "%s (%s)"
	REQUEST.form["browseSortKey"] = "displayName"
	REQUEST.form["ids"] = []
	REQUEST.form["deleteBrowseKeys"] = " Delete "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( message == "" )
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getBrowseSortKey() == 'displayName' )
	self.failUnless( schema.getBrowseFormat() == '%s (%s)' )
	self.failUnless( schema.getBrowseKeys() == ['displayName'] )

    def test15(self):
        """ 15 TestZSchema: testing Org Unit Indicator form """

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaOrgUnitIndicator"
	REQUEST.form["personsOUAttr"] = "dn"
	REQUEST.form["searchOrgUnitIndicatorSave"] = " Save Changes "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( index(message,"Saved changes.") == 0 )
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getPersonsOUAttr() == "dn" )

	REQUEST = Request(sys.stdin, os.environ, self.response )
	REQUEST.form["updateSchema"] = 1
	REQUEST.form["objectCategory"] = "person"
	REQUEST.form["schemaFormID"] = "schemaOrgUnitIndicator"
	REQUEST.form["personsOUAttr"] = "useGivenAttr"
	REQUEST.form["givenAttr"] = "ou"
	REQUEST.form["searchOrgUnitIndicatorSave"] = " Save Changes "
        zschema = ZSchema( self.user, REQUEST )
        message = zschema.updateSchemaFromRequest()
	self.failUnless( index(message,"Saved changes.") == 0 )
        schema = zschema.getManagedSchema()
	self.failUnless( schema.getPersonsOUAttr() == "ou" )

    def tearDown(self):
        self.user.unbind()

if __name__ == '__main__':

    tests = unittest.makeSuite(testZSchema, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
